@@ -16,27 +16,23 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include <config.h>
 
-#ifdef HAVE_UNISTD_H
-#   include <unistd.h>
-#endif
-
+#include <errno.h>
 #include <stdio.h>
+#include <string.h>
+
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <string.h>
-#include <errno.h>
-
-#ifdef USE_NETCODE
-#include <netdb.h>
-#endif
-
 #ifdef HAVE_MMAP
 #   include <sys/mman.h>
 #endif
+#ifdef USE_NETCODE
+#include <netdb.h>
+#endif
+#include <unistd.h>
 
 #include "global.h"
 #include "cmd.h"		/* Our definitions */
@@ -48,15 +44,14 @@
 #include "subshell.h"		/* use_subshell */
 #include "cons.saver.h"		/* console_flag */
 #include "tty.h"		/* LINES */
-#include "dlg.h"		/* Widget */
-#include "view.h"		/* view() */
+#include "dialog.h"		/* Widget */
+#include "view.h"		/* mc_internal_viewer() */
 #include "wtools.h"		/* message() */
 #include "widget.h"		/* push_history() */
 #include "key.h"		/* application_keypad_mode() */
 #include "win.h"		/* do_enter_ca_mode() */
-#include "dialog.h"		/* do_refresh() */
 #include "main.h"		/* change_panel() */
-#include "panel.h"		/* cpanel */
+#include "panel.h"		/* current_panel */
 #include "help.h"		/* interactive_display() */
 #include "user.h"		/* MC_GLOBAL_MENU */
 #include "command.h"		/* cmdline */
@@ -65,6 +60,7 @@
 #include "boxes.h"		/* cd_dialog() */
 #include "setup.h"		/* save_setup() */
 #include "profile.h"		/* PROFILE_NAME */
+#include "execute.h"		/* toggle_panels() */
 
 #ifndef MAP_FILE
 #   define MAP_FILE 0
@@ -77,62 +73,15 @@
 /* If set and you don't have subshell support,then C-o will give you a shell */
 int output_starts_shell = 0;
 
-/* Source routing destination */
-int source_route = 0;
-
 /* If set, use the builtin editor */
 int use_internal_edit = 1;
 
 
-/*
- * Execute command on a filename that can be on VFS.
- * Errors are reported to the user.
- */
-static void
-execute_with_vfs_arg (const char *command, const char *filename)
-{
-    char *localcopy;
-    char *fn;
-    struct stat st;
-    time_t mtime;
-
-    /* Simplest case, this file is local */
-    if (vfs_file_is_local (filename)) {
-	execute_internal (command, filename);
-	return;
-    }
-
-    /* FIXME: Creation of new files on VFS is not supported */
-    if (!*filename)
-	return;
-
-    localcopy = mc_getlocalcopy (filename);
-    if (localcopy == NULL) {
-	message (1, MSG_ERROR, _(" Cannot fetch a local copy of %s "),
-		 filename);
-	return;
-    }
-
-    /*
-     * filename can be an entry on panel, it can be changed by executing
-     * the command, so make a copy.  Smarter VFS code would make the code
-     * below unnecessary.
-     */
-    fn = g_strdup (filename);
-    mc_stat (localcopy, &st);
-    mtime = st.st_mtime;
-    execute_internal (command, localcopy);
-    mc_stat (localcopy, &st);
-    mc_ungetlocalcopy (fn, localcopy, mtime != st.st_mtime);
-    g_free (fn);
-}
-
-
 int
-view_file_at_line (char *filename, int plain_view, int internal,
+view_file_at_line (const char *filename, int plain_view, int internal,
 		   int start_line)
 {
-    static char *viewer = 0;
+    static const char *viewer = NULL;
     int move_dir = 0;
 
 
@@ -153,7 +102,7 @@ view_file_at_line (char *filename, int plain_view, int internal,
 	default_hex_mode = 0;
 	default_nroff_flag = 0;
 	default_magic_flag = 0;
-	view (0, filename, &move_dir, start_line);
+	mc_internal_viewer (NULL, filename, &move_dir, start_line);
 	if (changed_hex_mode && !altered_hex_mode)
 	    default_hex_mode = 1;
 	if (changed_nroff_flag && !altered_nroff_flag)
@@ -173,12 +122,14 @@ view_file_at_line (char *filename, int plain_view, int internal,
 	    strcpy (view_entry, "View");
 
 	if (regex_command (filename, view_entry, &move_dir) == 0) {
-	    view (0, filename, &move_dir, start_line);
+	    mc_internal_viewer (NULL, filename, &move_dir, start_line);
 	    repaint_screen ();
 	}
     } else {
 	if (!viewer) {
-	    viewer = getenv ("PAGER");
+	    viewer = getenv ("VIEWER");
+	    if (!viewer)
+		viewer = getenv ("PAGER");
 	    if (!viewer)
 		viewer = "view";
 	}
@@ -196,7 +147,7 @@ view_file_at_line (char *filename, int plain_view, int internal,
  *   internal:   If set uses the internal viewer, otherwise an external viewer.
  */
 int
-view_file (char *filename, int plain_view, int internal)
+view_file (const char *filename, int plain_view, int internal)
 {
     return view_file_at_line (filename, plain_view, internal, 0);
 }
@@ -217,7 +168,7 @@ static int scan_for_file (WPanel *panel, int idx, int direction)
 	    i = panel->count - 1;
 	if (i == panel->count)
 	    i = 0;
-	if (!S_ISDIR (panel->dir.list [i].buf.st_mode))
+	if (!S_ISDIR (panel->dir.list [i].st.st_mode))
 	    return i;
 	i += direction;
     }
@@ -234,32 +185,32 @@ do_view_cmd (int normal)
     int dir, file_idx;
 
     /* Directories are viewed by changing to them */
-    if (S_ISDIR (selection (cpanel)->buf.st_mode)
-	|| link_isdir (selection (cpanel))) {
-	if (confirm_view_dir && (cpanel->marked || cpanel->dirs_marked)) {
+    if (S_ISDIR (selection (current_panel)->st.st_mode)
+	|| link_isdir (selection (current_panel))) {
+	if (confirm_view_dir && (current_panel->marked || current_panel->dirs_marked)) {
 	    if (query_dialog
 		(_(" Confirmation "), _("Files tagged, want to cd?"), 0, 2,
 		 _("&Yes"), _("&No")) != 0) {
 		return;
 	    }
 	}
-	if (!do_cd (selection (cpanel)->fname, cd_exact))
+	if (!do_cd (selection (current_panel)->fname, cd_exact))
 	    message (1, MSG_ERROR, _("Cannot change directory"));
 
 	return;
 
     }
 
-    file_idx = cpanel->selected;
+    file_idx = current_panel->selected;
     while (1) {
 	char *filename;
 
-	filename = cpanel->dir.list[file_idx].fname;
+	filename = current_panel->dir.list[file_idx].fname;
 
 	dir = view_file (filename, normal, use_internal_view);
 	if (dir == 0)
 	    break;
-	file_idx = scan_for_file (cpanel, file_idx, dir);
+	file_idx = scan_for_file (current_panel, file_idx, dir);
     }
 }
 
@@ -277,8 +228,8 @@ view_file_cmd (void)
     char *filename;
 
     filename =
-	input_dialog (_(" View file "), _(" Filename:"),
-		      selection (cpanel)->fname);
+	input_expand_dialog (_(" View file "), _(" Filename:"),
+			     selection (current_panel)->fname);
     if (!filename)
 	return;
 
@@ -301,22 +252,22 @@ filtered_view_cmd (void)
     command =
 	input_dialog (_(" Filtered view "),
 		      _(" Filter command and arguments:"),
-		      selection (cpanel)->fname);
+		      selection (current_panel)->fname);
     if (!command)
 	return;
 
-    view (command, "", 0, 0);
+    mc_internal_viewer (command, "", NULL, 0);
 
     g_free (command);
 }
 
 void do_edit_at_line (const char *what, int start_line)
 {
-    static char *editor = 0;
+    static const char *editor = NULL;
 
 #ifdef USE_INTERNAL_EDIT
     if (use_internal_edit){
-	edit (what, start_line);
+	edit_file (what, start_line);
 	update_panels (UP_OPTIMIZE, UP_KEEPSEL);
 	repaint_screen ();
 	return;
@@ -341,14 +292,14 @@ do_edit (const char *what)
 void
 edit_cmd (void)
 {
-    if (regex_command (selection (cpanel)->fname, "Edit", 0) == 0)
-	do_edit (selection (cpanel)->fname);
+    if (regex_command (selection (current_panel)->fname, "Edit", 0) == 0)
+	do_edit (selection (current_panel)->fname);
 }
 
 void
 edit_cmd_new (void)
 {
-    do_edit ("");
+    do_edit (NULL);
 }
 
 /* Invoked by F5.  Copy, default to the other panel.  */
@@ -356,27 +307,27 @@ void
 copy_cmd (void)
 {
     save_cwds_stat ();
-    if (panel_operate (cpanel, OP_COPY, NULL, TRUE)) {
+    if (panel_operate (current_panel, OP_COPY, 0)) {
 	update_panels (UP_OPTIMIZE, UP_KEEPSEL);
 	repaint_screen ();
     }
 }
 
-/* Invoked by F6.  Move/rename, default to the other panel.  */
+/* Invoked by F6.  Move/rename, default to the other panel, ignore marks.  */
 void ren_cmd (void)
 {
     save_cwds_stat ();
-    if (panel_operate (cpanel, OP_MOVE, NULL, TRUE)){
+    if (panel_operate (current_panel, OP_MOVE, 0)){
 	update_panels (UP_OPTIMIZE, UP_KEEPSEL);
 	repaint_screen ();
     }
 }
 
-/* Invoked by F15.  Copy, default to the same panel.  */
+/* Invoked by F15.  Copy, default to the same panel, ignore marks.  */
 void copy_cmd_local (void)
 {
     save_cwds_stat ();
-    if (panel_operate (cpanel, OP_COPY, selection (cpanel)->fname, TRUE)){
+    if (panel_operate (current_panel, OP_COPY, 1)){
 	update_panels (UP_OPTIMIZE, UP_KEEPSEL);
 	repaint_screen ();
     }
@@ -386,7 +337,7 @@ void copy_cmd_local (void)
 void ren_cmd_local (void)
 {
     save_cwds_stat ();
-    if (panel_operate (cpanel, OP_MOVE, selection (cpanel)->fname, TRUE)){
+    if (panel_operate (current_panel, OP_MOVE, 1)){
 	update_panels (UP_OPTIMIZE, UP_KEEPSEL);
 	repaint_screen ();
     }
@@ -395,39 +346,48 @@ void ren_cmd_local (void)
 void
 mkdir_cmd (void)
 {
-    char *tempdir;
-    char *dir;
+    char *dir, *absdir;
 
     dir =
 	input_expand_dialog (_("Create a new Directory"),
 			     _(" Enter directory name:"), "");
-
-    if (!dir || !*dir)
+    if (!dir)
 	return;
 
     if (dir[0] == '/' || dir[0] == '~')
-	tempdir = g_strdup (dir);
+	absdir = g_strdup (dir);
     else
-	tempdir = concat_dir_and_file (cpanel->cwd, dir);
-    g_free (dir);
+	absdir = concat_dir_and_file (current_panel->cwd, dir);
 
     save_cwds_stat ();
-    if (my_mkdir (tempdir, 0777) == 0) {
-	update_panels (UP_OPTIMIZE, tempdir);
+    if (my_mkdir (absdir, 0777) == 0) {
+	update_panels (UP_OPTIMIZE, dir);
 	repaint_screen ();
-	select_item (cpanel);
-	g_free (tempdir);
-	return;
+	select_item (current_panel);
+    } else {
+	message (1, MSG_ERROR, "  %s  ", unix_error_string (errno));
     }
-    g_free (tempdir);
-    message (1, MSG_ERROR, "  %s  ", unix_error_string (errno));
+
+    g_free (absdir);
+    g_free (dir);
 }
 
 void delete_cmd (void)
 {
     save_cwds_stat ();
 
-    if (panel_operate (cpanel, OP_DELETE, NULL, TRUE)){
+    if (panel_operate (current_panel, OP_DELETE, 0)){
+	update_panels (UP_OPTIMIZE, UP_KEEPSEL);
+	repaint_screen ();
+    }
+}
+
+/* Invoked by F18.  Remove selected file, regardless of marked files.  */
+void delete_cmd_local (void)
+{
+    save_cwds_stat ();
+
+    if (panel_operate (current_panel, OP_DELETE, 1)){
 	update_panels (UP_OPTIMIZE, UP_KEEPSEL);
 	repaint_screen ();
     }
@@ -441,10 +401,9 @@ void find_cmd (void)
 static void
 set_panel_filter_to (WPanel *p, char *allocated_filter_string)
 {
-    if (p->filter){
-	g_free (p->filter);
-	p->filter = 0;
-    }
+    g_free (p->filter);
+    p->filter = 0;
+
     if (!(allocated_filter_string [0] == '*' && allocated_filter_string [1] == 0))
 	p->filter = allocated_filter_string;
     else
@@ -457,7 +416,7 @@ static void
 set_panel_filter (WPanel *p)
 {
     char *reg_exp;
-    char *x;
+    const char *x;
     
     x = p->filter ? p->filter : easy_patterns ? "*" : ".";
 	
@@ -487,7 +446,7 @@ void reread_cmd (void)
 
     if (get_current_type () == view_listing &&
 	get_other_type () == view_listing)
-	flag = strcmp (cpanel->cwd, opanel->cwd) ? UP_ONLY_CURRENT : 0;
+	flag = strcmp (current_panel->cwd, other_panel->cwd) ? UP_ONLY_CURRENT : 0;
     else
 	flag = UP_ONLY_CURRENT;
 	
@@ -500,106 +459,74 @@ void reverse_selection_cmd (void)
     file_entry *file;
     int i;
 
-    for (i = 0; i < cpanel->count; i++){
-	file = &cpanel->dir.list [i];
-	if (S_ISDIR (file->buf.st_mode))
+    for (i = 0; i < current_panel->count; i++){
+	file = &current_panel->dir.list [i];
+	if (S_ISDIR (file->st.st_mode))
 	    continue;
-	do_file_mark (cpanel, i, !file->f.marked);
+	do_file_mark (current_panel, i, !file->f.marked);
     }
-    paint_panel (cpanel);
+}
+
+static void
+select_unselect_cmd (const char *title, int cmd)
+{
+    char *reg_exp, *reg_exp_t;
+    int i;
+    int c;
+    int dirflag = 0;
+
+    reg_exp = input_dialog (title, "", easy_patterns ? "*" : ".");
+    if (!reg_exp)
+	return;
+    if (!*reg_exp) {
+	g_free (reg_exp);
+	return;
+    }
+
+    reg_exp_t = reg_exp;
+
+    /* Check if they specified a directory */
+    if (*reg_exp_t == PATH_SEP) {
+	dirflag = 1;
+	reg_exp_t++;
+    }
+    if (reg_exp_t[strlen (reg_exp_t) - 1] == PATH_SEP) {
+	dirflag = 1;
+	reg_exp_t[strlen (reg_exp_t) - 1] = 0;
+    }
+
+    for (i = 0; i < current_panel->count; i++) {
+	if (!strcmp (current_panel->dir.list[i].fname, ".."))
+	    continue;
+	if (S_ISDIR (current_panel->dir.list[i].st.st_mode)) {
+	    if (!dirflag)
+		continue;
+	} else {
+	    if (dirflag)
+		continue;
+	}
+	c = regexp_match (reg_exp_t, current_panel->dir.list[i].fname,
+			  match_file);
+	if (c == -1) {
+	    message (1, MSG_ERROR, _("  Malformed regular expression  "));
+	    g_free (reg_exp);
+	    return;
+	}
+	if (c) {
+	    do_file_mark (current_panel, i, cmd);
+	}
+    }
+    g_free (reg_exp);
 }
 
 void select_cmd (void)
 {
-    char *reg_exp, *reg_exp_t;
-    int i;
-    int c;
-    int dirflag = 0;
-
-    reg_exp = input_dialog (_(" Select "), "", easy_patterns ? "*" : ".");
-    if (!reg_exp)
-	return;
-    
-    reg_exp_t = reg_exp;
-
-    /* Check if they specified a directory */
-    if (*reg_exp_t == PATH_SEP){
-        dirflag = 1;
-        reg_exp_t++;
-    }
-    if (reg_exp_t [strlen(reg_exp_t) - 1] == PATH_SEP){
-        dirflag = 1;
-        reg_exp_t [strlen(reg_exp_t) - 1] = 0;
-    }
-
-    for (i = 0; i < cpanel->count; i++){
-        if (!strcmp (cpanel->dir.list [i].fname, ".."))
-            continue;
-	if (S_ISDIR (cpanel->dir.list [i].buf.st_mode)){
-	    if (!dirflag)
-                continue;
-        } else {
-            if (dirflag)
-                continue;
-	}
-	c = regexp_match (reg_exp_t, cpanel->dir.list [i].fname, match_file);
-	if (c == -1){
-	    message (1, MSG_ERROR, _("  Malformed regular expression  "));
-	    g_free (reg_exp);
-	    return;
-	}
-	if (c){
-	    do_file_mark (cpanel, i, 1);
-	}
-    }
-    paint_panel (cpanel);
-    g_free (reg_exp);
+    select_unselect_cmd (_(" Select "), 1);
 }
 
 void unselect_cmd (void)
 {
-    char *reg_exp, *reg_exp_t;
-    int i;
-    int c;
-    int dirflag = 0;
-
-    reg_exp = input_dialog (_(" Unselect "),"", easy_patterns ? "*" : ".");
-    if (!reg_exp)
-	return;
-    
-    reg_exp_t = reg_exp;
-    
-    /* Check if they specified directory matching */
-    if (*reg_exp_t == PATH_SEP){
-        dirflag = 1;
-	reg_exp_t ++;
-    }
-    if (reg_exp_t [strlen(reg_exp_t) - 1] == PATH_SEP){
-        dirflag = 1;
-        reg_exp_t [strlen(reg_exp_t) - 1] = 0;
-    }
-    for (i = 0; i < cpanel->count; i++){
-        if (!strcmp (cpanel->dir.list [i].fname, "..")) 
-            continue;
-	if (S_ISDIR (cpanel->dir.list [i].buf.st_mode)){
-	    if (!dirflag)
-	        continue;
-        } else {
-            if (dirflag)
-                continue;
-        }
-	c = regexp_match (reg_exp_t, cpanel->dir.list [i].fname, match_file);
-	if (c == -1){
-	    message (1, MSG_ERROR, _("  Malformed regular expression  "));
-	    g_free (reg_exp);
-	    return;
-	}
-	if (c){
-	    do_file_mark (cpanel, i, 0);
-	}
-    }
-    paint_panel (cpanel);
-    g_free (reg_exp);
+    select_unselect_cmd (_(" Unselect "), 0);
 }
 
 /* Check if the file exists */
@@ -616,8 +543,8 @@ check_for_default(char *default_file, char *file)
 	if (mc_stat (default_file, &s)){
 	    return -1;
 	}
-	ctx = file_op_context_new ();
-	file_op_context_create_ui (ctx, OP_COPY, 0);
+	ctx = file_op_context_new (OP_COPY);
+	file_op_context_create_ui (ctx, 0);
 	copy_file_file (ctx, default_file, file, 1, &count, &bytes, 1);
 	file_op_context_destroy (ctx);
     }
@@ -754,11 +681,6 @@ edit_syntax_cmd (void)
 #endif
 
 #ifdef USE_VFS
-void free_vfs_now (void)
-{
-    vfs_expire (1);
-}
-
 void reselect_vfs (void)
 {
     char *target;
@@ -837,7 +759,7 @@ compare_dir (WPanel *panel, WPanel *other, enum CompareMode mode)
 	file_mark (panel, i, 0);
 
 	/* Skip directories */
-	if (S_ISDIR (source->buf.st_mode))
+	if (S_ISDIR (source->st.st_mode))
 	    continue;
 
 	/* Search the corresponding entry from the other panel */
@@ -855,12 +777,12 @@ compare_dir (WPanel *panel, WPanel *other, enum CompareMode mode)
 
 	    if (mode != compare_size_only){
 		/* Older version is not marked */
-		if (source->buf.st_mtime < target->buf.st_mtime)
+		if (source->st.st_mtime < target->st.st_mtime)
 		    continue;
 	    }
 	    
 	    /* Newer version with different size is marked */
-	    if (source->buf.st_size != target->buf.st_size){
+	    if (source->st.st_size != target->st.st_size){
 		do_file_mark (panel, i, 1);
 		continue;
 		
@@ -871,7 +793,7 @@ compare_dir (WPanel *panel, WPanel *other, enum CompareMode mode)
 	    if (mode == compare_quick){
 		/* Thorough compare off, compare only time stamps */
 		/* Mark newer version, don't mark version with the same date */
-		if (source->buf.st_mtime > target->buf.st_mtime){
+		if (source->st.st_mtime > target->st.st_mtime){
 		    do_file_mark (panel, i, 1);
 		}
 		continue;
@@ -880,7 +802,7 @@ compare_dir (WPanel *panel, WPanel *other, enum CompareMode mode)
 	    /* Thorough compare on, do byte-by-byte comparison */
 	    src_name = concat_dir_and_file (panel->cwd, source->fname);
 	    dst_name = concat_dir_and_file (other->cwd, target->fname);
-	    if (compare_files (src_name, dst_name, source->buf.st_size))
+	    if (compare_files (src_name, dst_name, source->st.st_size))
 		do_file_mark (panel, i, 1);
 	    g_free (src_name);
 	    g_free (dst_name);
@@ -906,10 +828,8 @@ compare_dirs_cmd (void)
 
     if (get_current_type () == view_listing
 	&& get_other_type () == view_listing) {
-	compare_dir (cpanel, opanel, thorough_flag);
-	compare_dir (opanel, cpanel, thorough_flag);
-	paint_panel (cpanel);
-	paint_panel (opanel);
+	compare_dir (current_panel, other_panel, thorough_flag);
+	compare_dir (other_panel, current_panel, thorough_flag);
     } else {
 	message (1, MSG_ERROR,
 		 _(" Both panels should be in the "
@@ -921,7 +841,7 @@ void
 history_cmd (void)
 {
     Listbox *listbox;
-    Hist *current;
+    GList *current;
 
     if (cmdline->need_push) {
 	if (push_history (cmdline, cmdline->buffer) == 2)
@@ -931,14 +851,12 @@ history_cmd (void)
 	message (1, MSG_ERROR, _(" The command history is empty "));
 	return;
     }
-    current = cmdline->history;
-    while (current->prev)
-	current = current->prev;
+    current = g_list_first (cmdline->history);
     listbox = create_listbox_window (60, 10, _(" Command history "),
 				     "[Command Menu]");
     while (current) {
-	LISTBOX_APPEND_TEXT (listbox, 0, current->text, current);
-	current = current->next;
+	LISTBOX_APPEND_TEXT (listbox, 0, (char *) current->data, current);
+	current = g_list_next(current);
     }
     run_dlg (listbox->dlg);
     if (listbox->dlg->ret_value == B_CANCEL)
@@ -951,7 +869,7 @@ history_cmd (void)
     if (!current)
 	return;
     cmdline->history = current;
-    assign_text (cmdline, cmdline->history->text);
+    assign_text (cmdline, (char *) current->data);
     update_input (cmdline, 1);
 }
 
@@ -966,106 +884,28 @@ void
 view_other_cmd (void)
 {
     static int message_flag = TRUE;
-#ifdef HAVE_SUBSHELL_SUPPORT
-    char *new_dir = NULL;
-    char **new_dir_p;
-#endif /* HAVE_SUBSHELL_SUPPORT */
 
-    if (!xterm_flag && !console_flag && !use_subshell){
+    if (!xterm_flag && !console_flag && !use_subshell && !output_starts_shell) {
 	if (message_flag)
-	    message (1, MSG_ERROR, _(" Not an xterm or Linux console; \n"
-				     " the panels cannot be toggled. "));
+	    message (1, MSG_ERROR,
+		     _(" Not an xterm or Linux console; \n"
+		       " the panels cannot be toggled. "));
 	message_flag = FALSE;
     } else {
-	channels_down ();
-	disable_mouse ();
-	if (clear_before_exec)
-	    clr_scr ();
-        if (alternate_plus_minus)
-            numeric_keypad_mode ();
-#ifndef HAVE_SLANG
-	/* With slang we don't want any of this, since there
-	 * is no mc_raw_mode supported
-	 */
-	reset_shell_mode ();
-	noecho ();
-#endif /* !HAVE_SLANG */
-	keypad(stdscr, FALSE);
-	endwin ();
-	do_exit_ca_mode ();
-	mc_raw_mode ();
-	if (console_flag)
-	    restore_console ();
-
-#ifdef HAVE_SUBSHELL_SUPPORT
-	if (use_subshell){
-	    new_dir_p = vfs_current_is_local () ? &new_dir : NULL;
-	    if (invoke_subshell (NULL, VISIBLY, new_dir_p))
-		quiet_quit_cmd();  /* User did `exit' or `logout': quit MC quietly */
-	} else
-#endif /* HAVE_SUBSHELL_SUPPORT */
-	{
-	    if (output_starts_shell){
-		fprintf (stderr,
-		 _("Type `exit' to return to the Midnight Commander"));
-		fprintf (stderr, "\n\r\n\r");
-			 
-		my_system (EXECUTE_AS_SHELL, shell, NULL);
-	    } else
-		get_key_code (0);
-	}
-	if (console_flag)
-	    handle_console (CONSOLE_SAVE);
-
-	do_enter_ca_mode ();
-
-	reset_prog_mode ();
-	keypad(stdscr, TRUE);
-
-	/* Prevent screen flash when user did 'exit' or 'logout' within
-	   subshell */
-	if (quit)
-	    return;
-
-	enable_mouse ();
-	channels_up ();
-        if (alternate_plus_minus)
-            application_keypad_mode ();
-
-#ifdef HAVE_SUBSHELL_SUPPORT
-	if (use_subshell){
-	    load_prompt (0, 0);
-	    if (new_dir)
-		do_possible_cd (new_dir);
-	    if (console_flag && output_lines)
-		show_console_contents (output_start_y,
-				       LINES-keybar_visible-output_lines-1,
-				       LINES-keybar_visible-1);
-	}
-#endif /* HAVE_SUBSHELL_SUPPORT */
-        touchwin (stdscr);
-	
-	repaint_screen ();
-	update_xterm_title_path ();
+	toggle_panels ();
     }
 }
 
-#ifndef NATIVE_WIN32
 static void
-do_link (int symbolic_link, char *fname)
+do_link (int symbolic_link, const char *fname)
 {
-    char *dest, *src;
+    char *dest = NULL, *src = NULL;
 
     if (!symbolic_link) {
 	src = g_strdup_printf (_("Link %s to:"), name_trunc (fname, 46));
 	dest = input_expand_dialog (_(" Link "), src, "");
-	g_free (src);
-	if (!dest)
-	    return;
-	if (!*dest) {
-	    g_free (dest);
-	    return;
-	}
+	if (!dest || !*dest)
+	    goto cleanup;
 	save_cwds_stat ();
 	if (-1 == mc_link (fname, dest))
 	    message (1, MSG_ERROR, _(" link: %s "),
@@ -1075,10 +915,10 @@ do_link (int symbolic_link, char *fname)
 	char *d;
 
 	/* suggest the full path for symlink */
-	s = concat_dir_and_file (cpanel->cwd, fname);
+	s = concat_dir_and_file (current_panel->cwd, fname);
 
 	if (get_other_type () == view_listing) {
-	    d = concat_dir_and_file (opanel->cwd, fname);
+	    d = concat_dir_and_file (other_panel->cwd, fname);
 	} else {
 	    d = g_strdup (fname);
 	}
@@ -1087,33 +927,30 @@ do_link (int symbolic_link, char *fname)
 	g_free (d);
 	g_free (s);
 
-	if (!dest || !*dest || !src || !*src) {
-	    if (src)
-		g_free (src);
-	    if (dest)
-		g_free (dest);
-	    return;
-	}
+	if (!dest || !*dest || !src || !*src)
+	    goto cleanup;
 	save_cwds_stat ();
 	if (-1 == mc_symlink (dest, src))
 	    message (1, MSG_ERROR, _(" symlink: %s "),
 		     unix_error_string (errno));
-	g_free (src);
     }
-    g_free (dest);
     update_panels (UP_OPTIMIZE, UP_KEEPSEL);
     repaint_screen ();
+
+cleanup:
+    g_free (src);
+    g_free (dest);
 }
 
 void link_cmd (void)
 {
-    do_link (0, selection (cpanel)->fname);
+    do_link (0, selection (current_panel)->fname);
 }
 
 void symlink_cmd (void)
 {
     char *filename = NULL;
-    filename = selection (cpanel)->fname;
+    filename = selection (current_panel)->fname;
 
     if (filename) {
 	do_link (1, filename);
@@ -1122,17 +959,17 @@ void symlink_cmd (void)
 
 void edit_symlink_cmd (void)
 {
-    if (S_ISLNK (selection (cpanel)->buf.st_mode)) {
+    if (S_ISLNK (selection (current_panel)->st.st_mode)) {
 	char buffer [MC_MAXPATHLEN];
 	char *p = NULL;
 	int i;
 	char *dest, *q;
 
-	p = selection (cpanel)->fname;
+	p = selection (current_panel)->fname;
 
 	q = g_strdup_printf (_(" Symlink `%s\' points to: "), name_trunc (p, 32));
 
-	i = readlink (p, buffer, MC_MAXPATHLEN);
+	i = readlink (p, buffer, MC_MAXPATHLEN - 1);
 	if (i > 0) {
 	    buffer [i] = 0;
 	    dest = input_expand_dialog (_(" Edit symlink "), q, buffer);
@@ -1156,16 +993,21 @@ void edit_symlink_cmd (void)
 	g_free (q);
     } else {
 	message (1, MSG_ERROR, _("`%s' is not a symbolic link"),
-		 selection (cpanel)->fname);
+		 selection (current_panel)->fname);
     }
 }
-#endif /* !NATIVE_WIN32 */
 
 void help_cmd (void)
 {
    interactive_display (NULL, "[main]");
 }
 
+void
+user_file_menu_cmd (void)
+{
+    user_menu_cmd (NULL);
+}
+
 /* partly taken from dcigettext.c, returns "" for default locale */
 /* value should be freed by calling function g_free() */
 char *guess_message_value (void)
@@ -1185,7 +1027,7 @@ char *guess_message_value (void)
     };
 
     unsigned i = 0;
-    char *locale = NULL;
+    const char *locale = NULL;
 
     while (var[i] != NULL) {
 	locale = getenv (var[i]);
@@ -1207,59 +1049,43 @@ char *
 get_random_hint (int force)
 {
     char *data, *result, *eol;
-    int  len;
+    int len;
     int start;
-    
-    /* Do not change hints more often than one minute */
-
-#ifdef SCO_FLAVOR
-    static time_t last;
-    time_t now;
-
-    time (&now);
-    if (!force && (now - last) < 60)
-	return g_strdup ("");
-    last = now;
-#else
     static int last_sec;
     static struct timeval tv;
-    
+
+    /* Do not change hints more often than one minute */
     gettimeofday (&tv, NULL);
     if (!force && !(tv.tv_sec > last_sec + 60))
-	return g_strdup (""); 
+	return g_strdup ("");
     last_sec = tv.tv_sec;
-#endif /* !SCO_FLAVOR */
 
     data = load_mc_home_file (MC_HINT, NULL);
     if (!data)
 	return 0;
 
-#ifdef SCO_FLAVOR
-    srand ((short) now);
-#else
-    srand (tv.tv_sec);
-#endif /* !SCO_FLAVOR */
     /* get a random entry */
+    srand (tv.tv_sec);
     len = strlen (data);
     start = rand () % len;
-    
-    for (;start; start--){
-	if (data [start] == '\n'){
+
+    for (; start; start--) {
+	if (data[start] == '\n') {
 	    start++;
 	    break;
 	}
     }
-    eol = strchr (&data [start], '\n');
+    eol = strchr (&data[start], '\n');
     if (eol)
 	*eol = 0;
-    result = g_strdup (&data [start]);
+    result = g_strdup (&data[start]);
     g_free (data);
     return result;
 }
 
 #if defined(USE_NETCODE) || defined(USE_EXT2FSLIB)
 static void
-nice_cd (char *text, char *xtext, char *help, char *prefix, int to_home)
+nice_cd (const char *text, const char *xtext, const char *help, const char *prefix, int to_home)
 {
     char *machine;
     char *cd_path;
@@ -1277,9 +1103,9 @@ nice_cd (char *text, char *xtext, char *help, char *prefix, int to_home)
 			   ugly as hell and leads to problems in vfs layer */
 
     if (strncmp (prefix, machine, strlen (prefix)) == 0)
-	cd_path = g_strconcat (machine, to_home ? "/~/" : NULL, NULL);
+	cd_path = g_strconcat (machine, to_home ? "/~/" : (char *) NULL, (char *) NULL);
     else 
-	cd_path = g_strconcat (prefix, machine, to_home ? "/~/" : NULL, NULL);
+	cd_path = g_strconcat (prefix, machine, to_home ? "/~/" : (char *) NULL, (char *) NULL);
     
     if (do_panel_cd (MENU_PANEL, cd_path, 0))
 	directory_history_add (MENU_PANEL, (MENU_PANEL)->cwd);
@@ -1293,7 +1119,7 @@ nice_cd (char *text, char *xtext, char *help, char *prefix, int to_home)
 
 #ifdef USE_NETCODE
 
-static char *machine_str = N_(" Enter machine name (F1 for details): ");
+static const char *machine_str = N_(" Enter machine name (F1 for details): ");
 
 #ifdef WITH_MCFS
 void netlink_cmd (void)
@@ -1322,28 +1148,6 @@ void smblink_cmd (void)
 	     "[SMB File System]", "/#smb:", 0);
 }
 #endif /* WITH_SMBFS */
-
-#ifdef HAVE_SETSOCKOPT
-void source_routing (void)
-{
-    char *source;
-    struct hostent *hp;
-    
-    source = input_dialog (_(" Socket source routing setup "),
-			   _(" Enter host name to use as a source routing hop: "),
-			   "");
-    if (!source)
-	return;
-
-    hp = gethostbyname (source);
-    g_free (source);
-    if (!hp){
-	message (1, _(" Host name "), _(" Error while looking up IP address "));
-	return;
-    }
-    source_route = *((int *)hp->h_addr);
-}
-#endif /* HAVE_SETSOCKOPT */
 #endif /* USE_NETCODE */
 
 #ifdef USE_EXT2FSLIB
@@ -1361,36 +1165,58 @@ void quick_cd_cmd (void)
     char *p = cd_dialog ();
 
     if (p && *p) {
-        char *q = g_strconcat ("cd ", p, NULL);
+        char *q = g_strconcat ("cd ", p, (char *) NULL);
         
         do_cd_command (q);
         g_free (q);
     }
-    if (p)
-       g_free (p);
+    g_free (p);
+}
+
+void
+single_dirsize_cmd (void)
+{
+    WPanel *panel = current_panel;
+    file_entry *entry;
+    off_t marked;
+    double total;
+
+    entry = &(panel->dir.list[panel->selected]);
+    if (S_ISDIR (entry->st.st_mode) && strcmp(entry->fname, "..") != 0) {
+	total = 0.0;
+	compute_dir_size (entry->fname, &marked, &total);
+	entry->st.st_size = (off_t) total;
+	entry->f.dir_size_computed = 1;
+    }
+
+    if (mark_moves_down)
+	send_message (&(panel->widget), WIDGET_KEY, KEY_DOWN);
+
+    recalculate_panel_summary (panel);
+    panel->dirty = 1;
 }
 
 void 
 dirsizes_cmd (void)
 {
-    WPanel *panel = cpanel;
+    WPanel *panel = current_panel;
     int i;
     off_t marked;
     double total;
 
     for (i = 0; i < panel->count; i++) 
-	if (S_ISDIR (panel->dir.list [i].buf.st_mode) &&
+	if (S_ISDIR (panel->dir.list [i].st.st_mode) &&
 	         ((panel->dirs_marked && panel->dir.list [i].f.marked) || 
                    !panel->dirs_marked) &&
 	         strcmp (panel->dir.list [i].fname, "..") != 0) {
 	    total = 0.0l;
 	    compute_dir_size (panel->dir.list [i].fname, &marked, &total);
-	    panel->dir.list [i].buf.st_size = (off_t) total;
+	    panel->dir.list [i].st.st_size = (off_t) total;
 	    panel->dir.list [i].f.dir_size_computed = 1;
 	}
 	
     recalculate_panel_summary (panel);
-    paint_panel (panel);
+    panel->dirty = 1;
 }
 
 void
@@ -1400,7 +1226,7 @@ save_setup_cmd (void)
     
     save_setup ();
     sync_profiles ();
-    str = g_strconcat ( _(" Setup saved to ~/"), PROFILE_NAME, NULL);
+    str = g_strconcat ( _(" Setup saved to ~/"), PROFILE_NAME, (char *) NULL);
     
     message (0, _(" Setup "), str);
     g_free (str);
@@ -1427,8 +1253,6 @@ configure_panel_listing (WPanel *p, int view_type, int use_msformat, char *user,
     }
 
     set_panel_formats (p);
-    paint_panel (p);
-    
     do_refresh ();
 }
 
@@ -1440,7 +1264,7 @@ info_cmd_no_menu (void)
     else if (get_display_type (1) == view_info)
 	set_display_type (1, view_listing);
     else
-	set_display_type (cpanel == left_panel ? 1 : 0, view_info);
+	set_display_type (current_panel == left_panel ? 1 : 0, view_info);
 }
 
 void
@@ -1451,7 +1275,7 @@ quick_cmd_no_menu (void)
     else if (get_display_type (1) == view_quick)
 	set_display_type (1, view_listing);
     else
-	set_display_type (cpanel == left_panel ? 1 : 0, view_quick);
+	set_display_type (current_panel == left_panel ? 1 : 0, view_quick);
 }
 
 static void
@@ -1502,7 +1326,7 @@ info_cmd (void)
 void
 quick_view_cmd (void)
 {
-    if ((WPanel *) get_panel_widget (MENU_PANEL_IDX) == cpanel)
+    if ((WPanel *) get_panel_widget (MENU_PANEL_IDX) == current_panel)
 	change_panel ();
     set_display_type (MENU_PANEL_IDX, view_quick);
 }
@@ -1518,7 +1342,6 @@ set_basic_panel_listing_to (int panel_index, int listing_mode)
     if (set_panel_formats (p))
 	return 0;
 	
-    paint_panel (p);
     do_refresh ();
     return 1;
 }
@@ -1528,22 +1351,7 @@ toggle_listing_cmd (void)
 {
     int current = get_current_index ();
     WPanel *p = (WPanel *) get_panel_widget (current);
-    int list_mode = p->list_type;
-    int m;
-    
-    switch (list_mode){
-    case list_full:
-    case list_brief:
-	m = list_long;
-	break;
-    case list_long:
-	m = list_user;
-	break;
-    default:
-	m = list_full;
-    }
-    if (set_basic_panel_listing_to (current, m))
-	return;
-    set_basic_panel_listing_to (current, list_full);
+
+    set_basic_panel_listing_to (current, (p->list_type + 1) % LIST_TYPES);
 }
 
