@@ -1,37 +1,29 @@
-#ifndef __MAIN_H
-#define __MAIN_H
+#ifndef MC_MAIN_H
+#define MC_MAIN_H
+
+#include "menu.h"
+#include "panel.h"
+#include "widget.h"
+
 /* Toggling functions */
 void toggle_fast_reload (void);
 void toggle_mix_all_files (void);
 void toggle_show_backup (void);
 void toggle_show_hidden (void);
 
-enum {
-    RP_NOCLEAR,
-    RP_CLEAR
-};
-
 #define UP_OPTIMIZE 0
 #define UP_RELOAD   1
 #define UP_ONLY_CURRENT  2
 
-#define UP_KEEPSEL (char *) -1
+#define UP_KEEPSEL ((char *) -1)
 
 extern int quote;
 extern volatile int quit;
 
-/* Execute functions: the base and the routines that use it */
-void do_execute (const char *shell, const char *command, int internal_command);
-#define execute_internal(command,args) do_execute (command, args, 1)
+/* If true, after executing a command, wait for a keystroke */
+enum { pause_never, pause_on_dumb_terminals, pause_always };
 
-/* Execute functions that use the shell to execute */
-void shell_execute (char *command, int flags);
-void execute (char *command);
-
-/* This one executes a shell */
-void exec_shell (void);
-
-void subshell_chdir (char *command);
+void subshell_chdir (const char *command);
 
 /* See main.c for details on these variables */
 extern int mark_moves_down;
@@ -40,8 +32,6 @@ extern int pause_after_run;
 extern int auto_save_setup;
 extern int use_internal_view;
 extern int use_internal_edit;
-extern char *view_one_file;
-extern char *edit_one_file;
 extern int fast_reload_w;
 extern int clear_before_exec;
 extern int mou_auto_repeat;
@@ -65,24 +55,22 @@ extern int cd_symlinks;
 extern int show_all_if_ambiguous;
 extern int slow_terminal;
 extern int update_prompt;	/* To comunicate with subshell */
+extern int safe_delete;
 extern int confirm_delete;
+extern int confirm_directory_hotlist_delete;
 extern int confirm_execute;
 extern int confirm_exit;
 extern int confirm_overwrite;
 extern int force_colors;
 extern int boot_current_is_left;
-extern int acs_vline;
-extern int acs_hline;
 extern int use_file_to_check_type;
-extern int searching;
 extern int vfs_use_limit;
 extern int alternate_plus_minus;
 extern int only_leading_plus_minus;
 extern int output_starts_shell;
 extern int midnight_shutdown;
-extern char search_buffer [256];
 extern char cmd_buf [512];
-extern char *cmdline_geometry;
+extern const char *shell;
 
 /* Ugly hack in order to distinguish between left and right panel in menubar */
 extern int is_right;		/* If the selected menu was the right */
@@ -90,10 +78,7 @@ extern int is_right;		/* If the selected menu was the right */
 #define MENU_PANEL_IDX  (is_right ? 1 : 0)
 #define SELECTED_IS_PANEL (get_display_type (is_right ? 1 : 0) == view_listing)
 
-typedef void (*key_callback) ();
-/* FIXME: We have to leave this type ambiguous, because `key_callback'
-   is used both for functions that take an argument and ones that don't.
-   That ought to be cleared up. */
+typedef void (*key_callback) (void);
 
 /* The keymaps are of this type */
 typedef struct {
@@ -101,74 +86,51 @@ typedef struct {
     key_callback fn;
 } key_map;
 
-void update_panels (int force_update, char *current_file);
-void create_panels (void);
+void update_panels (int force_update, const char *current_file);
 void repaint_screen (void);
-void outrefresh_screen (void);
-void suspend_cmd (void);
 void do_update_prompt (void);
 
-extern char *shell;
-
-/* directory specified on command line for startup */
-extern char *this_dir;
-
-void clr_scr (void);
-void restore_console (void);
-
 enum cd_enum {
     cd_parse_command,
     cd_exact
 };
 
-int do_cd           (char *new_dir, enum cd_enum cd_type);	/* For find.c */
+int do_cd           (const char *new_dir, enum cd_enum cd_type); /* For find.c */
 void change_panel   (void);
-void init_sigchld   (void);	/* For subshell.c */
-int load_prompt     (int fd, void *unused);
+int load_prompt     (int, void *);
 void save_cwds_stat (void);
-int quiet_quit_cmd  (void);	/* For cmd.c and command.c */
+void quiet_quit_cmd (void);	/* For cmd.c and command.c */
 
-void untouch_bar    (void);
 void touch_bar      (void);
 void update_xterm_title_path (void);
 void load_hint      (int force);
 
-void print_vfs_message(char *msg, ...)
-    __attribute__ ((format (printf, 1, 2)));
+void print_vfs_message(const char *msg, ...)
+    __attribute__ ((format (__printf__, 1, 2)));
 
-extern char *prompt;
+extern const char *prompt;
+extern const char *edit_one_file;
 extern char *mc_home;
 char *get_mc_lib_dir (void);
 
-int maybe_cd (int char_code, int move_up_dir);
-void do_possible_cd (char *dir);
+int maybe_cd (int move_up_dir);
+void do_possible_cd (const char *dir);
 
 #ifdef WANT_WIDGETS
 extern WButtonBar *the_bar;
 extern WLabel     *the_prompt;
 extern WLabel     *the_hint;
 extern Dlg_head   *midnight_dlg;
-extern Dlg_head   *edit_dlg;
-extern WLabel     *process_status;
 
-struct WMenu;
 extern struct WMenu *the_menubar;
 #endif /* WANT_WIDGETS */
 
-void edition_pre_exec (void);
-void edition_post_exec (void);
-
 void done_menu (void);
 void init_menu (void);
 
-#ifdef NATIVE_WIN32
-#    define MC_BASE ""
-#else
-#    define MC_BASE "/.mc/"
-#endif /* !NATIVE_WIN32 */
+#define MC_BASE "/.mc/"
 
-struct WPanel;
-void directory_history_add   (struct WPanel *panel, char *s);
-int  do_panel_cd             (struct WPanel *panel, char *new_dir, enum cd_enum cd_type);
+void directory_history_add   (struct WPanel *panel, const char *dir);
+int  do_panel_cd             (struct WPanel *panel, const char *new_dir, enum cd_enum cd_type);
 
-#endif /* !__MAIN_H */
+#endif
