@@ -13,28 +13,16 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
-#ifndef NO_INFOMOUNT
 
 #include <stdio.h>
-#include <sys/types.h>
-#include "mountlist.h"
-
-#ifdef STDC_HEADERS
 #include <stdlib.h>
-#else
-void free (void *ptr);
-#endif
-#if defined(STDC_HEADERS) || defined(HAVE_STRING_H)
-#include <string.h>
-#else
-#include <strings.h>
-#endif
 
+#include <sys/types.h>
+
+/* This header needs to be included before sys/mount.h on *BSD */
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
@@ -60,6 +48,10 @@ void free (void *ptr);
 #include <sys/mount.h>
 #endif
 
+#ifdef MOUNTED_GETMNTINFO2	/* NetBSD 3.0.  */
+#include <sys/statvfs.h>
+#endif
+
 #ifdef MOUNTED_GETMNT		/* Ultrix.  */
 #include <sys/mount.h>
 #include <sys/fs_types.h>
@@ -84,7 +76,38 @@ void free (void *ptr);
 #include <sys/vfs.h>
 #endif
 
-/* void error (void);  FIXME -- needed? */
+#ifdef HAVE_SYS_STATFS_H
+#include <sys/statfs.h>
+#endif
+
+#ifdef HAVE_INFOMOUNT_QNX
+#include <sys/disk.h>
+#include <sys/fsys.h>
+#endif
+
+#ifdef HAVE_SYS_MOUNT_H
+#include <sys/mount.h>
+#endif
+
+#ifdef HAVE_SYS_VFS_H
+#include <sys/vfs.h>
+#endif
+
+#ifdef HAVE_SYS_FILSYS_H
+#include <sys/filsys.h>		/* SVR2.  */
+#endif
+
+#ifdef HAVE_DUSTAT_H		/* AIX PS/2.  */
+#include <sys/dustat.h>
+#endif
+
+#ifdef HAVE_SYS_STATVFS_H	/* SVR4.  */
+#include <sys/statvfs.h>
+#endif
+
+#include "global.h"
+#include "mountlist.h"
+#include "util.h"
 
 #ifdef DOLPHIN
 /* So special that it's not worth putting this in autoconf.  */
@@ -92,12 +115,45 @@ void free (void *ptr);
 #define MOUNTED_GETMNTTBL
 #endif
 
+#if defined (__QNX__) && !defined(__QNXNTO__) && !defined (HAVE_INFOMOUNT_LIST)
+#    define HAVE_INFOMOUNT_QNX
+#endif
+
+#if defined(HAVE_INFOMOUNT_LIST) || defined(HAVE_INFOMOUNT_QNX)
+#    define HAVE_INFOMOUNT
+#endif
+
+/* A mount table entry. */
+struct mount_entry
+{
+  char *me_devname;		/* Device node pathname, including "/dev/". */
+  char *me_mountdir;		/* Mount point directory pathname. */
+  char *me_type;		/* "nfs", "4.2", etc. */
+  dev_t me_dev;			/* Device number of me_mountdir. */
+  struct mount_entry *me_next;
+};
+
+struct fs_usage
+{
+  long fsu_blocks;		/* Total blocks. */
+  long fsu_bfree;		/* Free blocks available to superuser. */
+  long fsu_bavail;		/* Free blocks available to non-superuser. */
+  long fsu_files;		/* Total file nodes. */
+  long fsu_ffree;		/* Free file nodes. */
+};
+
+static int get_fs_usage (char *path, struct fs_usage *fsp);
+
+#ifdef HAVE_INFOMOUNT_LIST
+
+static struct mount_entry *mount_list = NULL;
+
 #ifdef MOUNTED_GETMNTENT1	/* 4.3BSD, SunOS, HP-UX, Dynix, Irix.  */
 /* Return the value of the hexadecimal number represented by CP.
    No prefix (like '0x') or suffix (like 'h') is expected to be
    part of CP. */
 
-static int xatoi (char *cp)
+static int xatoi (const char *cp)
 {
     int val;
 
@@ -115,16 +171,12 @@ static int xatoi (char *cp)
     }
     return val;
 }
-#endif				/* MOUNTED_GETMNTENT1.  */
+#endif /* MOUNTED_GETMNTENT1 */
 
-#if defined (MOUNTED_GETMNTINFO) && !defined (HAVE_F_FSTYPENAME)
+#ifndef HAVE_STRUCT_STATFS_F_FSTYPENAME
 static char *fstype_to_string (short t)
 {
     switch (t) {
-    case MOUNT_UFS:
-	return "ufs";
-    case MOUNT_NFS:
-	return "nfs";
 #ifdef MOUNT_PC
     case MOUNT_PC:
 	return "pc";
@@ -144,15 +196,80 @@ static char *fstype_to_string (short t)
 #ifdef MOUNT_TMP
     case MOUNT_TMP:
 	return "tmp";
+#endif
+#ifdef MOUNT_UFS
+   case MOUNT_UFS:
+     return "ufs" ;
+#endif
+#ifdef MOUNT_NFS
+   case MOUNT_NFS:
+     return "nfs" ;
+#endif
+#ifdef MOUNT_MSDOS
+   case MOUNT_MSDOS:
+     return "msdos" ;
+#endif
+#ifdef MOUNT_LFS
+   case MOUNT_LFS:
+     return "lfs" ;
+#endif
+#ifdef MOUNT_LOFS
+   case MOUNT_LOFS:
+     return "lofs" ;
+#endif
+#ifdef MOUNT_FDESC
+   case MOUNT_FDESC:
+     return "fdesc" ;
+#endif
+#ifdef MOUNT_PORTAL
+   case MOUNT_PORTAL:
+     return "portal" ;
+#endif
+#ifdef MOUNT_NULL
+   case MOUNT_NULL:
+     return "null" ;
+#endif
+#ifdef MOUNT_UMAP
+   case MOUNT_UMAP:
+     return "umap" ;
+#endif
+#ifdef MOUNT_KERNFS
+   case MOUNT_KERNFS:
+     return "kernfs" ;
+#endif
+#ifdef MOUNT_PROCFS
+   case MOUNT_PROCFS:
+     return "procfs" ;
+#endif
+#ifdef MOUNT_AFS
+   case MOUNT_AFS:
+     return "afs" ;
+#endif
+#ifdef MOUNT_CD9660
+   case MOUNT_CD9660:
+     return "cd9660" ;
+#endif
+#ifdef MOUNT_UNION
+   case MOUNT_UNION:
+     return "union" ;
+#endif
+#ifdef MOUNT_DEVFS
+   case MOUNT_DEVFS:
+     return "devfs" ;
+#endif
+#ifdef MOUNT_EXT2FS
+   case MOUNT_EXT2FS:
+     return "ext2fs" ;
 #endif
     default:
 	return "?";
     }
 }
-#endif				/* MOUNTED_GETMNTINFO */
+#endif /* ! HAVE_STRUCT_STATFS_F_FSTYPENAME */
 
 #ifdef MOUNTED_VMOUNT		/* AIX.  */
-static char *fstype_to_string (int t)
+static char *
+fstype_to_string (int t)
 {
     struct vfs_ent *e;
 
@@ -162,7 +279,7 @@ static char *fstype_to_string (int t)
     else
 	return e->vfsent_name;
 }
-#endif				/* MOUNTED_VMOUNT */
+#endif /* MOUNTED_VMOUNT */
 
 /* Return a list of the currently mounted filesystems, or NULL on error.
    Add each entry to the tail of the list so that they stay in order.
@@ -171,25 +288,29 @@ static char *fstype_to_string (int t)
    If ALL_FS is zero, do not return entries for filesystems that
    are automounter (dummy) entries.  */
 
-struct mount_entry *read_filesystem_list (int need_fs_type, int all_fs)
+static struct mount_entry *
+read_filesystem_list (int need_fs_type, int all_fs)
 {
-    struct mount_entry *mount_list;
+    struct mount_entry *mlist;
     struct mount_entry *me;
     struct mount_entry *mtail;
 
+    (void) need_fs_type;
+    (void) all_fs;
+
     /* Start the list off with a dummy entry. */
     me = (struct mount_entry *) malloc (sizeof (struct mount_entry));
     me->me_next = NULL;
-    mount_list = mtail = me;
+    mlist = mtail = me;
 
 #ifdef MOUNTED_GETMNTENT1	/* 4.3BSD, SunOS, HP-UX, Dynix, Irix.  */
+#ifdef MOUNTED
     {
 	struct mntent *mnt;
-	char *table = MOUNTED;
 	FILE *fp;
-	char *devopt;
+	const char *devopt;
 
-	fp = setmntent (table, "r");
+	fp = setmntent (MOUNTED, "r");
 	if (fp == NULL)
 	    return NULL;
 
@@ -220,7 +341,8 @@ struct mount_entry *read_filesystem_list (int need_fs_type, int all_fs)
 	if (endmntent (fp) == 0)
 	    return NULL;
     }
-#endif				/* MOUNTED_GETMNTENT1. */
+#endif /* MOUNTED */
+#endif /* MOUNTED_GETMNTENT1 */
 
 #ifdef MOUNTED_GETMNTINFO	/* 4.4BSD.  */
     {
@@ -234,7 +356,7 @@ struct mount_entry *read_filesystem_list (int need_fs_type, int all_fs)
 	    me = (struct mount_entry *) malloc (sizeof (struct mount_entry));
 	    me->me_devname = strdup (fsp->f_mntfromname);
 	    me->me_mountdir = strdup (fsp->f_mntonname);
-#ifdef HAVE_F_FSTYPENAME
+#ifdef HAVE_STRUCT_STATFS_F_FSTYPENAME
 	    me->me_type = strdup (fsp->f_fstypename);
 #else
 	    me->me_type = fstype_to_string (fsp->f_type);
@@ -248,7 +370,29 @@ struct mount_entry *read_filesystem_list (int need_fs_type, int all_fs)
 	    fsp++;
 	}
     }
-#endif				/* MOUNTED_GETMNTINFO */
+#endif /* MOUNTED_GETMNTINFO */
+
+#ifdef MOUNTED_GETMNTINFO2	/* NetBSD 3.0.  */
+    {
+	struct statvfs *fsp;
+	int entries;
+
+	entries = getmntinfo (&fsp, MNT_NOWAIT);
+	if (entries < 0)
+	    return NULL;
+	for (; entries-- > 0; fsp++) {
+	    me = (struct mount_entry *) malloc (sizeof (struct mount_entry));
+	    me->me_devname = strdup (fsp->f_mntfromname);
+	    me->me_mountdir = strdup (fsp->f_mntonname);
+	    me->me_type = strdup (fsp->f_fstypename);
+	    me->me_dev = (dev_t) -1;	/* Magic; means not known yet. */
+
+	    /* Add to the linked list. */
+	    mtail->me_next = me;
+	    mtail = me;
+	}
+    }
+#endif /* MOUNTED_GETMNTINFO2 */
 
 #ifdef MOUNTED_GETMNT		/* Ultrix.  */
     {
@@ -257,7 +401,7 @@ struct mount_entry *read_filesystem_list (int need_fs_type, int all_fs)
 	struct fs_data fsd;
 
 	while ((val = getmnt (&offset, &fsd, sizeof (fsd), NOSTAT_MANY,
-			      (char *) 0)) > 0) {
+			      NULL)) > 0) {
 	    me = (struct mount_entry *) malloc (sizeof (struct mount_entry));
 	    me->me_devname = strdup (fsd.fd_req.devname);
 	    me->me_mountdir = strdup (fsd.fd_req.path);
@@ -272,9 +416,9 @@ struct mount_entry *read_filesystem_list (int need_fs_type, int all_fs)
 	if (val < 0)
 	    return NULL;
     }
-#endif				/* MOUNTED_GETMNT. */
+#endif /* MOUNTED_GETMNT */
 
-#if defined (MOUNTED_GETFSSTAT)	/* __alpha running OSF_1 */
+#ifdef MOUNTED_GETFSSTAT	/* __alpha running OSF_1 */
     {
 	int numsys, counter, bufsize;
 	struct statfs *stats;
@@ -306,7 +450,7 @@ struct mount_entry *read_filesystem_list (int need_fs_type, int all_fs)
 
 	free (stats);
     }
-#endif				/* MOUNTED_GETFSSTAT */
+#endif /* MOUNTED_GETFSSTAT */
 
 #if defined (MOUNTED_FREAD) || defined (MOUNTED_FREAD_FSTYP)	/* SVR[23].  */
     {
@@ -350,7 +494,7 @@ struct mount_entry *read_filesystem_list (int need_fs_type, int all_fs)
 	if (fclose (fp) == EOF)
 	    return NULL;
     }
-#endif				/* MOUNTED_FREAD || MOUNTED_FREAD_FSTYP.  */
+#endif /* MOUNTED_FREAD || MOUNTED_FREAD_FSTYP */
 
 #ifdef MOUNTED_GETMNTTBL	/* DolphinOS goes it's own way */
     {
@@ -369,7 +513,7 @@ struct mount_entry *read_filesystem_list (int need_fs_type, int all_fs)
 	}
 	endmnttbl ();
     }
-#endif
+#endif /* MOUNTED_GETMNTTBL */
 
 #ifdef MOUNTED_GETMNTENT2	/* SVR4.  */
     {
@@ -399,7 +543,7 @@ struct mount_entry *read_filesystem_list (int need_fs_type, int all_fs)
 	if (fclose (fp) == EOF)
 	    return NULL;
     }
-#endif				/* MOUNTED_GETMNTENT2.  */
+#endif /* MOUNTED_GETMNTENT2 */
 
 #ifdef MOUNTED_VMOUNT		/* AIX.  */
     {
@@ -443,17 +587,17 @@ struct mount_entry *read_filesystem_list (int need_fs_type, int all_fs)
 	}
 	free (entries);
     }
-#endif				/* MOUNTED_VMOUNT. */
+#endif /* MOUNTED_VMOUNT */
 
     /* Free the dummy head. */
-    me = mount_list;
-    mount_list = mount_list->me_next;
+    me = mlist;
+    mlist = mlist->me_next;
     free (me);
-    return mount_list;
+    return mlist;
 }
-#endif /* NO_INFOMOUNT */
+#endif /* HAVE_INFOMOUNT_LIST */
 
-#if defined(NO_INFOMOUNT) && defined(__QNX__)
+#ifdef HAVE_INFOMOUNT_QNX
 /*
 ** QNX has no [gs]etmnt*(), [gs]etfs*(), or /etc/mnttab, but can do
 ** this via the following code.
@@ -462,20 +606,8 @@ struct mount_entry *read_filesystem_list (int need_fs_type, int all_fs)
 ** this hack.
 */
 
-#include <fcntl.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#ifdef HAVE_UNISTD_H
-#    include <unistd.h>
-#endif
-#include <sys/disk.h>
-#include <sys/fsys.h>
-#include <sys/statfs.h>
-
-#include "mountlist.h"
-
-struct mount_entry *read_filesystem_list(int need_fs_type, int all_fs)
+static struct mount_entry *
+read_filesystem_list(int need_fs_type, int all_fs)
 {
 	struct _disk_entry	de;
 	struct statfs		fs;
@@ -529,9 +661,221 @@ struct mount_entry *read_filesystem_list(int need_fs_type, int all_fs)
 		de.disk_type, tp, _DRIVER_NAME_LEN, _DRIVER_NAME_LEN, de.driver_name, de.disk_drv);
 	fprintf(stderr, "fsys_get_mount_dev():\n\tdevice='%s'\n", dev);
 	fprintf(stderr, "fsys_get_mount_pt():\n\tmount point='%s'\n", dir);
-#endif
+#endif /* DEBUG */
 
 	return (me);
 }
-#endif /* __QNX__ */
+#endif /* HAVE_INFOMOUNT_QNX */
 
+void
+init_my_statfs (void)
+{
+#ifdef HAVE_INFOMOUNT_LIST
+    mount_list = read_filesystem_list (1, 1);
+#endif /* HAVE_INFOMOUNT_LIST */
+}
+
+void
+my_statfs (struct my_statfs *myfs_stats, const char *path)
+{
+#ifdef HAVE_INFOMOUNT_LIST
+    int i, len = 0;
+    struct mount_entry *entry = NULL;
+    struct mount_entry *temp = mount_list;
+    struct fs_usage fs_use;
+
+    while (temp){
+	i = strlen (temp->me_mountdir);
+	if (i > len && (strncmp (path, temp->me_mountdir, i) == 0))
+	    if (!entry || (path [i] == PATH_SEP || path [i] == 0)){
+		len = i;
+		entry = temp;
+	    }
+	temp = temp->me_next;
+    }
+
+    if (entry){
+	memset (&fs_use, 0, sizeof (struct fs_usage));
+	get_fs_usage (entry->me_mountdir, &fs_use);
+
+	myfs_stats->type = entry->me_dev;
+	myfs_stats->typename = entry->me_type;
+	myfs_stats->mpoint = entry->me_mountdir;
+	myfs_stats->device = entry->me_devname;
+	myfs_stats->avail = getuid () ? fs_use.fsu_bavail/2 : fs_use.fsu_bfree/2;
+	myfs_stats->total = fs_use.fsu_blocks/2;
+	myfs_stats->nfree = fs_use.fsu_ffree;
+	myfs_stats->nodes = fs_use.fsu_files;
+    } else
+#endif /* HAVE_INFOMOUNT_LIST */
+
+#ifdef HAVE_INFOMOUNT_QNX
+/*
+** This is the "other side" of the hack to read_filesystem_list() in
+** mountlist.c.
+** It's not the most efficient approach, but consumes less memory. It
+** also accomodates QNX's ability to mount filesystems on the fly.
+*/
+	struct mount_entry	*entry;
+    struct fs_usage		fs_use;
+
+	if ((entry = read_filesystem_list(0, 0)) != NULL)
+	{
+		get_fs_usage(entry->me_mountdir, &fs_use);
+
+		myfs_stats->type = entry->me_dev;
+		myfs_stats->typename = entry->me_type;
+		myfs_stats->mpoint = entry->me_mountdir;
+		myfs_stats->device = entry->me_devname;
+
+		myfs_stats->avail = fs_use.fsu_bfree / 2;
+		myfs_stats->total = fs_use.fsu_blocks / 2;
+		myfs_stats->nfree = fs_use.fsu_ffree;
+		myfs_stats->nodes = fs_use.fsu_files;
+	}
+	else
+#endif /* HAVE_INFOMOUNT_QNX */
+    {
+	myfs_stats->type = 0;
+	myfs_stats->mpoint = "unknown";
+	myfs_stats->device = "unknown";
+	myfs_stats->avail = 0;
+	myfs_stats->total = 0;
+	myfs_stats->nfree = 0;
+	myfs_stats->nodes = 0;
+    }
+}
+
+#ifdef HAVE_INFOMOUNT
+
+/* Return the number of TOSIZE-byte blocks used by
+   BLOCKS FROMSIZE-byte blocks, rounding away from zero.
+   TOSIZE must be positive.  Return -1 if FROMSIZE is not positive.  */
+
+static long
+fs_adjust_blocks (long blocks, int fromsize, int tosize)
+{
+    if (tosize <= 0)
+	abort ();
+    if (fromsize <= 0)
+	return -1;
+
+    if (fromsize == tosize)	/* E.g., from 512 to 512.  */
+	return blocks;
+    else if (fromsize > tosize)	/* E.g., from 2048 to 512.  */
+	return blocks * (fromsize / tosize);
+    else			/* E.g., from 256 to 512.  */
+	return (blocks + (blocks < 0 ? -1 : 1)) / (tosize / fromsize);
+}
+
+#if defined(_AIX) && defined(_I386)
+/* AIX PS/2 does not supply statfs.  */
+static int aix_statfs (char *path, struct statfs *fsb)
+{
+    struct stat stats;
+    struct dustat fsd;
+
+    if (stat (path, &stats))
+	return -1;
+    if (dustat (stats.st_dev, 0, &fsd, sizeof (fsd)))
+	return -1;
+    fsb->f_type = 0;
+    fsb->f_bsize = fsd.du_bsize;
+    fsb->f_blocks = fsd.du_fsize - fsd.du_isize;
+    fsb->f_bfree = fsd.du_tfree;
+    fsb->f_bavail = fsd.du_tfree;
+    fsb->f_files = (fsd.du_isize - 2) * fsd.du_inopb;
+    fsb->f_ffree = fsd.du_tinode;
+    fsb->f_fsid.val[0] = fsd.du_site;
+    fsb->f_fsid.val[1] = fsd.du_pckno;
+    return 0;
+}
+#define statfs(path,fsb) aix_statfs(path,fsb)
+#endif				/* _AIX && _I386 */
+
+/* Fill in the fields of FSP with information about space usage for
+   the filesystem on which PATH resides.
+   Return 0 if successful, -1 if not. */
+
+static int
+get_fs_usage (char *path, struct fs_usage *fsp)
+{
+#ifdef STAT_STATFS3_OSF1
+    struct statfs fsd;
+
+    if (statfs (path, &fsd, sizeof (struct statfs)) != 0)
+	 return -1;
+#define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), fsd.f_fsize, 512)
+#endif				/* STAT_STATFS3_OSF1 */
+
+#ifdef STAT_STATFS2_FS_DATA	/* Ultrix.  */
+    struct fs_data fsd;
+
+    if (statfs (path, &fsd) != 1)
+	return -1;
+#define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), 1024, 512)
+    fsp->fsu_blocks = CONVERT_BLOCKS (fsd.fd_req.btot);
+    fsp->fsu_bfree = CONVERT_BLOCKS (fsd.fd_req.bfree);
+    fsp->fsu_bavail = CONVERT_BLOCKS (fsd.fd_req.bfreen);
+    fsp->fsu_files = fsd.fd_req.gtot;
+    fsp->fsu_ffree = fsd.fd_req.gfree;
+#endif
+
+#ifdef STAT_STATFS2_BSIZE	/* 4.3BSD, SunOS 4, HP-UX, AIX.  */
+    struct statfs fsd;
+
+    if (statfs (path, &fsd) < 0)
+	return -1;
+#define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), fsd.f_bsize, 512)
+#endif
+
+#ifdef STAT_STATFS2_FSIZE	/* 4.4BSD.  */
+    struct statfs fsd;
+
+    if (statfs (path, &fsd) < 0)
+	return -1;
+#define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), fsd.f_fsize, 512)
+#endif
+
+#ifdef STAT_STATFS4		/* SVR3, Dynix, Irix, AIX.  */
+    struct statfs fsd;
+
+    if (statfs (path, &fsd, sizeof fsd, 0) < 0)
+	return -1;
+    /* Empirically, the block counts on most SVR3 and SVR3-derived
+       systems seem to always be in terms of 512-byte blocks,
+       no matter what value f_bsize has.  */
+#if _AIX
+#define CONVERT_BLOCKS(b) fs_adjust_blocks ((b), fsd.f_bsize, 512)
+#else
+#define CONVERT_BLOCKS(b) (b)
+#ifndef _SEQUENT_		/* _SEQUENT_ is DYNIX/ptx.  */
+#ifndef DOLPHIN			/* DOLPHIN 3.8.alfa/7.18 has f_bavail */
+#define f_bavail f_bfree
+#endif
+#endif
+#endif
+#endif
+
+#ifdef STAT_STATVFS		/* SVR4.  */
+    struct statvfs fsd;
+
+    if (statvfs (path, &fsd) < 0)
+	return -1;
+    /* f_frsize isn't guaranteed to be supported.  */
+#define CONVERT_BLOCKS(b) \
+  fs_adjust_blocks ((b), fsd.f_frsize ? fsd.f_frsize : fsd.f_bsize, 512)
+#endif
+
+#if defined(CONVERT_BLOCKS) && !defined(STAT_STATFS2_FS_DATA) && !defined(STAT_READ_FILSYS)	/* !Ultrix && !SVR2.  */
+    fsp->fsu_blocks = CONVERT_BLOCKS (fsd.f_blocks);
+    fsp->fsu_bfree = CONVERT_BLOCKS (fsd.f_bfree);
+    fsp->fsu_bavail = CONVERT_BLOCKS (fsd.f_bavail);
+    fsp->fsu_files = fsd.f_files;
+    fsp->fsu_ffree = fsd.f_ffree;
+#endif
+
+    return 0;
+}
+
+#endif /* HAVE_INFOMOUNT */
