@@ -16,33 +16,30 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 #include <config.h>
-#ifdef HAVE_UNISTD_H
-#   include <unistd.h>
-#endif
-#include <string.h>
-#include <stdio.h>
+
 #include <errno.h>
+#include <stdio.h>
+#include <string.h>
+
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <unistd.h>
 
-/* Needed for the extern declarations of integer parameters */
-#define DIR_H_INCLUDE_HANDLE_DIRENT
 #include "global.h"
 #include "tty.h"		/* attrset() */
 #include "win.h"
 #include "color.h"
-#include "dlg.h"
+#include "dialog.h"
 #include "widget.h"
-#include "dialog.h"		/* For do_refresh() */
 #include "wtools.h"		/* For common_dialog_repaint() */
 #include "setup.h"		/* For profile_bname */
 #include "profile.h"		/* Load/save directories panelize */
 #include "dir.h"
-#include "panel.h"		/* cpanel */
+#include "panel.h"		/* current_panel */
 #include "main.h"		/* repaint_screen */
 #include "panelize.h"
 
@@ -55,7 +52,7 @@
 #define BUTTONS		4
 #define LABELS          3
 #define B_ADD		B_USER
-#define B_REMOVE        B_USER + 1
+#define B_REMOVE        (B_USER + 1)
 
 static WListbox *l_panelize;
 static Dlg_head *panelize_dlg;
@@ -64,16 +61,16 @@ static WInput *pname;
 
 static struct {
     int ret_cmd, flags, y, x;
-    char *text;
-    char *tkname;
+    const char *text;
 } panelize_but [BUTTONS] = {
-    { B_CANCEL, NORMAL_BUTTON, 0, 53, N_("&Cancel"),  "c"},
-    { B_ADD, NORMAL_BUTTON,    0, 28, N_("&Add new"), "a"},
-    { B_REMOVE, NORMAL_BUTTON, 0, 16, N_("&Remove"),  "r"},
-    { B_ENTER, DEFPUSH_BUTTON, 0,  0, N_("Pane&lize"),"l"},
+    { B_CANCEL, NORMAL_BUTTON, 0, 53, N_("&Cancel")   },
+    { B_ADD, NORMAL_BUTTON,    0, 28, N_("&Add new")  },
+    { B_REMOVE, NORMAL_BUTTON, 0, 16, N_("&Remove")   },
+    { B_ENTER, DEFPUSH_BUTTON, 0,  0, N_("Pane&lize") },
 };
 
-static char *panelize_section = "Panelize";
+static const char *panelize_section = "Panelize";
+static void do_external_panelize (char *command);
 
 /* Directory panelize */
 static struct panelize {
@@ -94,94 +91,98 @@ update_command (void)
     }
 }
 
-static int
-panelize_callback (Dlg_head * h, int Par, int Msg)
+static cb_ret_t
+panelize_callback (Dlg_head *h, dlg_msg_t msg, int parm)
 {
-    switch (Msg) {
+    switch (msg) {
     case DLG_DRAW:
 	common_dialog_repaint (h);
 	attrset (COLOR_NORMAL);
 	draw_box (h, UY, UX, h->lines - 10, h->cols - 10);
-	break;
+	return MSG_HANDLED;
 
     case DLG_POST_KEY:
 	/* fall */
     case DLG_INIT:
 	attrset (MENU_ENTRY_COLOR);
 	update_command ();
-	break;
+	return MSG_HANDLED;
+
+    default:
+	return default_dlg_callback (h, msg, parm);
     }
-    return 0;
 }
 
-static int l_call (void *data)
-{
-	return listbox_nothing;
-}
-
-static void init_panelize (void)
+static void
+init_panelize (void)
 {
     int i, panelize_cols = COLS - 6;
     struct panelize *current = panelize;
 
 #ifdef ENABLE_NLS
-	static int i18n_flag = 0;
-	static int maxlen = 0;
-	
-	if (!i18n_flag)
-	{
-		i = sizeof(panelize_but) / sizeof(panelize_but[0]);
-		while (i--)
-		{
-			panelize_but [i].text = _(panelize_but [i].text);
-			maxlen += strlen (panelize_but [i].text) + 5;
-		}
-		maxlen += 10;
+    static int i18n_flag = 0;
+    static int maxlen = 0;
 
-		i18n_flag = 1;
+    if (!i18n_flag) {
+	i = sizeof (panelize_but) / sizeof (panelize_but[0]);
+	while (i--) {
+	    panelize_but[i].text = _(panelize_but[i].text);
+	    maxlen += strlen (panelize_but[i].text) + 5;
 	}
-	panelize_cols = max(panelize_cols, maxlen);
-		
-	panelize_but [2].x = panelize_but [3].x 
-		+ strlen (panelize_but [3].text) + 7;
-	panelize_but [1].x = panelize_but [2].x 
-		+ strlen (panelize_but [2].text) + 5;
-	panelize_but [0].x = panelize_cols 
-		- strlen (panelize_but[0].text) - 8 - BX;
+	maxlen += 10;
+
+	i18n_flag = 1;
+    }
+    panelize_cols = max (panelize_cols, maxlen);
+
+    panelize_but[2].x =
+	panelize_but[3].x + strlen (panelize_but[3].text) + 7;
+    panelize_but[1].x =
+	panelize_but[2].x + strlen (panelize_but[2].text) + 5;
+    panelize_but[0].x =
+	panelize_cols - strlen (panelize_but[0].text) - 8 - BX;
+
+#endif				/* ENABLE_NLS */
 
-#endif /* ENABLE_NLS */
-    
     last_listitem = 0;
-    
+
     do_refresh ();
 
-    panelize_dlg = create_dlg (0, 0, 22, panelize_cols, dialog_colors,
-			      panelize_callback, "[External panelize]",
-			      _("External panelize"), DLG_CENTER);
-
-#define XTRACT(i) BY+panelize_but[i].y, BX+panelize_but[i].x, panelize_but[i].ret_cmd, panelize_but[i].flags, panelize_but[i].text, 0, 0, panelize_but[i].tkname
+    panelize_dlg =
+	create_dlg (0, 0, 22, panelize_cols, dialog_colors,
+		    panelize_callback, "[External panelize]",
+		    _("External panelize"), DLG_CENTER | DLG_REVERSE);
 
     for (i = 0; i < BUTTONS; i++)
-	add_widget(panelize_dlg, button_new (XTRACT (i)));
+	add_widget (panelize_dlg,
+		    button_new (BY + panelize_but[i].y,
+				BX + panelize_but[i].x,
+				panelize_but[i].ret_cmd,
+				panelize_but[i].flags,
+				panelize_but[i].text, 0));
 
-    pname = input_new (UY+14, UX, INPUT_COLOR, panelize_dlg->cols-10, "", "in");
+    pname =
+	input_new (UY + 14, UX, INPUT_COLOR, panelize_dlg->cols - 10, "",
+		   "in");
     add_widget (panelize_dlg, pname);
 
-    add_widget (panelize_dlg, label_new (UY+13, UX, _("Command"), "label-command"));
+    add_widget (panelize_dlg, label_new (UY + 13, UX, _("Command")));
 
     /* get new listbox */
-    l_panelize = listbox_new (UY + 1, UX + 1, panelize_dlg->cols-12, 10, 0, l_call, "li");
+    l_panelize =
+	listbox_new (UY + 1, UX + 1, panelize_dlg->cols - 12, 10, NULL);
 
-    while (current){
+    while (current) {
 	listbox_add_item (l_panelize, 0, 0, current->label, current);
 	current = current->next;
     }
 
     /* add listbox to the dialogs */
-    add_widget (panelize_dlg, l_panelize); 
+    add_widget (panelize_dlg, l_panelize);
 
-    listbox_select_entry (l_panelize, 
-        listbox_search_text (l_panelize, _("Other command")));
+    listbox_select_entry (l_panelize,
+			  listbox_search_text (l_panelize,
+					       _("Other command")));
 }
 
 static void panelize_done (void)
@@ -216,7 +217,7 @@ static void add2panelize (char *label, char *command)
     }
 }
 
-void
+static void
 add2panelize_cmd (void)
 {
     char *label;
@@ -352,12 +353,12 @@ void done_panelize (void)
     }
 }
 
-void do_external_panelize (char *command)
+static void do_external_panelize (char *command)
 {
     int status, link_to_dir, stale_link;
     int next_free = 0;
-    struct stat buf;
-    dir_list *list = &cpanel->dir;
+    struct stat st;
+    dir_list *list = &current_panel->dir;
     char line [MC_MAXPATHLEN];
     char *name;
     FILE *external;
@@ -369,7 +370,7 @@ void do_external_panelize (char *command)
 	return;
     }
     /* Clear the counters and the directory list */
-    panel_clean_dir (cpanel);
+    panel_clean_dir (current_panel);
 
     while (1) {
 	clearerr(external);
@@ -387,7 +388,7 @@ void do_external_panelize (char *command)
 	    name = line + 2;
 	else
 	    name = line;
-        status = handle_path (list, name, &buf, next_free, &link_to_dir,
+        status = handle_path (list, name, &st, next_free, &link_to_dir,
     	    &stale_link);
 	if (status == 0)
 	    continue;
@@ -395,34 +396,29 @@ void do_external_panelize (char *command)
 	    break;
 	list->list [next_free].fnamelen = strlen (name);
 	list->list [next_free].fname = g_strdup (name);
-	file_mark (cpanel, next_free, 0);
+	file_mark (current_panel, next_free, 0);
 	list->list [next_free].f.link_to_dir = link_to_dir;
 	list->list [next_free].f.stale_link = stale_link;
 	list->list [next_free].f.dir_size_computed = 0;
-	list->list [next_free].buf = buf;
+	list->list [next_free].st = st;
 	next_free++;
 	if (!(next_free & 32))
 	    rotate_dash ();
     }
 
-    cpanel->is_panelized = 1;
+    current_panel->is_panelized = 1;
     if (next_free){
-	cpanel->count = next_free;
+	current_panel->count = next_free;
 	if (list->list [0].fname [0] == PATH_SEP){
-	    strcpy (cpanel->cwd, PATH_SEP_STR);
+	    strcpy (current_panel->cwd, PATH_SEP_STR);
 	    chdir (PATH_SEP_STR);
 	}
     } else {
-	cpanel->count = set_zero_dir (list);
+	current_panel->count = set_zero_dir (list);
     }
-#ifndef SCO_FLAVOR
     if (pclose (external) < 0)
-#else /* SCO_FLAVOR */
-    if (WEXITSTATUS(pclose (external)) < 0)
-#endif /* SCO_FLAVOR */
 	message (0, _("External panelize"), _("Pipe close failed"));
     close_error_pipe (0, 0);
-    try_to_select (cpanel, NULL);
-    panel_re_sort (cpanel);
-    paint_panel (cpanel);
+    try_to_select (current_panel, NULL);
+    panel_re_sort (current_panel);
 }
