@@ -1,19 +1,11 @@
-#ifndef __WIDGET_H
-#define __WIDGET_H
+#ifndef MC_WIDGET_H
+#define MC_WIDGET_H
 
 #include "dialog.h"		/* Widget */
 
-#define C_BOOL		1
-#define C_CHANGE	2
-
 /* Please note that the first element in all the widgets is a     */
 /* widget variable of type Widget.  We abuse this fact everywhere */
 
-#define HIDDEN_BUTTON		0
-#define NARROW_BUTTON		1
-#define NORMAL_BUTTON		2
-#define DEFPUSH_BUTTON		3
-
 /* button callback */
 typedef int (*bcback) (int);
 
@@ -21,6 +13,11 @@ typedef struct WButton {
     Widget widget;
     int action;			/* what to do when pressed */
     int selected;		/* button state */
+
+#define HIDDEN_BUTTON		0
+#define NARROW_BUTTON		1
+#define NORMAL_BUTTON		2
+#define DEFPUSH_BUTTON		3
     unsigned int flags;		/* button flags */
     char *text;			/* text of button */
     int hotkey;			/* hot KEY */
@@ -34,14 +31,16 @@ typedef struct WRadio {
     int pos, sel;
     int count;			/* number of members */
     const char **texts;		/* texts of labels */
-    int upper_letter_is_hotkey; /* If true, then the capital letter is a hk */
 } WRadio;
 
 typedef struct WCheck {
     Widget widget;
+
+#define C_BOOL			0x0001
+#define C_CHANGE		0x0002
     unsigned int state;		/* check button state */
     char *text;			/* text of check button */
-    int hotkey;                 /* hot KEY */                    
+    int hotkey;                 /* hot KEY */
     int hotpos;			/* offset hot KEY char in text */
 } WCheck;
 
@@ -67,7 +66,7 @@ typedef struct {
     int  first;			/* Is first keystroke? */
     int  disable_update;	/* Do we want to skip updates? */
     int  is_password;		/* Is this a password input line? */
-    unsigned char *buffer;	/* pointer to editing buffer */
+    char *buffer;		/* pointer to editing buffer */
     GList *history;		/* The history */
     int  need_push;		/* need to push the current Input on hist? */
     char **completions;		/* Possible completions array */
@@ -77,7 +76,6 @@ typedef struct {
 
 /* For history load-save functions */
 #define INPUT_LAST_TEXT ((char *) 2)
-#define HISTORY_FILE_NAME ".mc/history"
 
 typedef struct {
     Widget widget;
@@ -124,22 +122,10 @@ typedef struct WGroupbox {
     char *title;
 } WGroupbox;
 
-typedef void (*buttonbarfn)(void *);
-
-typedef struct {
-    Widget widget;
-    int    visible;		/* Is it visible? */
-    struct {
-	char   *text;
-	buttonbarfn function;
-	void   *data;
-    } labels [10];
-} WButtonBar;
-
 /* Constructors */
 WButton *button_new   (int y, int x, int action, int flags, const char *text,
 		      bcback callback);
-WRadio  *radio_new    (int y, int x, int count, const char **text, int use_hotkey);
+WRadio  *radio_new    (int y, int x, int count, const char **text);
 WCheck  *check_new    (int y, int x, int state,  const char *text);
 WInput  *input_new    (int y, int x, int color, int len, const char *text, const char *histname);
 WLabel  *label_new    (int y, int x, const char *text);
@@ -161,7 +147,7 @@ void input_enable_update (WInput *in);
 void input_set_point (WInput *in, int pos);
 void input_show_cursor (WInput *in);
 void assign_text (WInput *in, const char *text);
-cb_ret_t input_callback (WInput *in, widget_msg_t msg, int parm);
+cb_ret_t input_callback (Widget *, widget_msg_t msg, int parm);
 
 /* Labels */
 void label_set_text (WLabel *label, const char *text);
@@ -171,6 +157,7 @@ void gauge_set_value (WGauge *g, int max, int current);
 void gauge_show (WGauge *g, int shown);
 
 /* Buttons */
+const char *button_get_text (WButton *b);
 void button_set_text (WButton *b, const char *text);
 
 /* Listbox manager */
@@ -196,14 +183,20 @@ char *listbox_add_item (WListbox *l, enum append_pos pos, int
 
 /* Hintbar routines */
 
-/* Buttonbar routines */
+/* Buttonbar */
+
+typedef void (*voidfn)(void);
+typedef void (*buttonbarfn)(void *);
+
+typedef struct WButtonBar WButtonBar;
+
 WButtonBar *buttonbar_new (int visible);
 WButtonBar *find_buttonbar (Dlg_head *h);
-typedef void (*voidfn)(void);
-void define_label (Dlg_head *, int index, const char *text, voidfn);
-void define_label_data (Dlg_head *h, int idx, const char *text,
-			buttonbarfn cback, void *data);
-void redraw_labels (Dlg_head *h);
-void buttonbar_hint (WButtonBar *bb, const char *s);
+void buttonbar_clear_label (Dlg_head *, int idx);
+void buttonbar_set_label (Dlg_head *, int index, const char *text, voidfn);
+void buttonbar_set_label_data (Dlg_head *h, int idx, const char *text,
+			       buttonbarfn cback, void *data);
+void buttonbar_set_visible (WButtonBar *, gboolean);
+void buttonbar_redraw (Dlg_head *h);
 
-#endif	/* __WIDGET_H */
+#endif
