@@ -18,7 +18,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
    TODO:
    opendir instead of keeping its table of file handles could return
@@ -32,12 +32,12 @@
 /* {{{ Includes and global variables */
 
 #include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #ifdef HAVE_UNISTD_H
 #    include <unistd.h>
 #endif
-#include <fcntl.h>
 #include <string.h>
 #ifdef HAVE_LIMITS_H
 #    include <limits.h>
@@ -51,27 +51,22 @@
 #ifdef HAVE_GRP_H
 #    include <grp.h>
 #endif
-#ifdef HAVE_SHADOW_H
-#    include <shadow.h>
-#else
-#    ifdef HAVE_SHADOW_SHADOW_H
-#        include <shadow/shadow.h>
-#    endif
-#endif
-#ifdef HAVE_CRYPT_H
-#    include <crypt.h>
-#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <errno.h>
 #include <signal.h>
+#ifdef HAVE_GETOPT_H
+#    include <getopt.h>
+#endif
 
 /* Network include files */
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
 #ifdef HAVE_PMAP_SET
 #    include <rpc/rpc.h>
 #    include <rpc/pmap_prot.h>
@@ -80,6 +75,12 @@
 #    endif
 #endif
 
+#if defined(HAVE_PAM)
+#  if !defined(HAVE_SECURITY_PAM_MISC_H)
+#    undef HAVE_PAM
+#  endif
+#endif
+
 /* Authentication include files */
 #include <pwd.h>
 #ifdef HAVE_PAM
@@ -87,14 +88,48 @@
 #    ifndef PAM_ESTABLISH_CRED
 #        define PAM_ESTABLISH_CRED PAM_CRED_ESTABLISH
 #    endif
+#else
+#endif				/* !HAVE_PAM */
+
+#ifdef HAVE_CRYPT_H
+#    include <crypt.h>
+#endif				/* !HAVE_CRYPT_H */
+
+#ifdef HAVE_SHADOW_H
+#    include <shadow.h>
+#else
+#    ifdef HAVE_SHADOW_SHADOW_H
+#        include <shadow/shadow.h>
+#    endif
 #endif
 
-#include "utilvfs.h"
+/*
+ * GNU gettext defines printf to libintl_printf on platforms that lack
+ * a native printf(3) capable of all POSIX features.
+ */
+#undef ENABLE_NLS
+#include "../src/global.h"
 
+#include "../src/tty.h"		/* enable/disable interrupt key */
+#include "../src/wtools.h"	/* message() */
+#include "../src/main.h"	/* print_vfs_message */
+#include "utilvfs.h"
 #include "vfs.h"
 #include "mcfs.h"
+#include "mcfsutil.h"
 #include "tcputil.h"
 
+#ifndef INADDR_NONE
+#  define INADDR_NONE (0xffffffffU)
+#endif
+
+/* replacement for g_free() from glib */
+#undef g_free
+#define g_free(x) do {if (x) free (x);} while (0)
+
+/* We don't care about SIGPIPE */
+int got_sigpipe = 0;
+
 /* The socket from which we accept commands */
 int msock;
 
@@ -105,15 +140,15 @@ static int clnt_version;
 int logged_in = 0;
 
 /* Home directory */
-char *home_dir = NULL;
+const char *home_dir = NULL;
 
 char *up_dir = NULL;
 
 /* Were we started from inetd? */
-int  inetd_started = 0;
+int inetd_started = 0;
 
 /* Are we running as a daemon? */
-int  isDaemon = 0;
+int isDaemon = 0;
 
 /* guess */
 int verbose = 0;
@@ -147,7 +182,7 @@ do { \
   return (a); \
 } while (0)
 
-char buffer [4096];
+char buffer[4096];
 int debug = 1;
 static int quit_server;
 static int return_code;
@@ -156,7 +191,8 @@ static int return_code;
 
 /* {{{ Misc routines */
 
-void send_status (int status, int errno_number)
+static void
+send_status (int status, int errno_number)
 {
     rpc_send (msock, RPC_INT, status, RPC_INT, errno_number, RPC_END);
     errno = 0;
@@ -166,43 +202,49 @@ void send_status (int status, int errno_number)
 
 /* {{{ File with handle operations */
 
-void do_open (void)
+static void
+do_open (void)
 {
     int handle, flags, mode;
     char *arg;
 
-    rpc_get (msock, RPC_STRING, &arg, RPC_INT, &flags, RPC_INT, &mode,RPC_END);
-    
+    rpc_get (msock, RPC_STRING, &arg, RPC_INT, &flags, RPC_INT, &mode,
+	     RPC_END);
+
     handle = open (arg, flags, mode);
     send_status (handle, errno);
     g_free (arg);
 }
 
-void do_read (void)
+static void
+do_read (void)
 {
     int handle, count, n;
     void *data;
-   
+
     rpc_get (msock, RPC_INT, &handle, RPC_INT, &count, RPC_END);
-    data = g_malloc (count);
-    if (!data){
+    data = malloc (count);
+    if (!data) {
 	send_status (-1, ENOMEM);
 	return;
     }
-    if (verbose) printf ("count=%d\n", count);
+    if (verbose)
+	printf ("count=%d\n", count);
     n = read (handle, data, count);
-    if (verbose) printf ("result=%d\n", n);
-    if (n < 0){
+    if (verbose)
+	printf ("result=%d\n", n);
+    if (n < 0) {
 	send_status (-1, errno);
 	return;
     }
     send_status (n, 0);
     rpc_send (msock, RPC_BLOCK, n, data, RPC_END);
-    
+
     g_free (data);
 }
 
-void do_write (void)
+static void
+do_write (void)
 {
     int handle, count, status, written = 0;
     char buf[8192];
@@ -229,22 +271,22 @@ void do_write (void)
     send_status (written, errno);
 }
 
-void do_lseek (void)
+static void
+do_lseek (void)
 {
     int handle, offset, whence, status;
 
-    rpc_get (msock,
-	     RPC_INT, &handle,
-	     RPC_INT, &offset,
-	     RPC_INT, &whence, RPC_END);
+    rpc_get (msock, RPC_INT, &handle, RPC_INT, &offset, RPC_INT, &whence,
+	     RPC_END);
     status = lseek (handle, offset, whence);
     send_status (status, errno);
 }
 
-void do_close (void)
+static void
+do_close (void)
 {
     int handle, status;
-    
+
     rpc_get (msock, RPC_INT, &handle, RPC_END);
     status = close (handle);
     send_status (status, errno);
@@ -254,87 +296,84 @@ void do_close (void)
 
 /* {{{ Stat family routines */
 
-void send_time (int sock, time_t time)
+static void
+send_time (int sock, time_t time)
 {
     if (clnt_version == 1) {
-	char   *ct;
-	int    month;
-    
+	char *ct;
+	int month;
+
 	ct = ctime (&time);
-	ct [3] = ct [10] = ct [13] = ct [16] = ct [19] = 0;
-	
+	ct[3] = ct[10] = ct[13] = ct[16] = ct[19] = 0;
+
 	/* Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec */
-	if (ct [4] == 'J'){
-	    if (ct [5] == 'a'){
+	if (ct[4] == 'J') {
+	    if (ct[5] == 'a') {
 		month = 0;
 	    } else
-		month = (ct [6] == 'n') ? 5 : 6;
-	} else if (ct [4] == 'F'){
+		month = (ct[6] == 'n') ? 5 : 6;
+	} else if (ct[4] == 'F') {
 	    month = 1;
-	} else if (ct [4] == 'M'){
-	    month = (ct [6] == 'r') ? 2 : 5;
-	} else if (ct [4] == 'A'){
-	    month = (ct [5] == 'p') ? 3 : 7;
-	} else if (ct [4] == 'S'){
+	} else if (ct[4] == 'M') {
+	    month = (ct[6] == 'r') ? 2 : 5;
+	} else if (ct[4] == 'A') {
+	    month = (ct[5] == 'p') ? 3 : 7;
+	} else if (ct[4] == 'S') {
 	    month = 8;
-	} else if (ct [4] == 'O'){
+	} else if (ct[4] == 'O') {
 	    month = 9;
-	} else if (ct [4] == 'N'){
+	} else if (ct[4] == 'N') {
 	    month = 10;
 	} else
-	month = 11;
-	rpc_send (msock,
-		  RPC_INT, atoi (&ct [17]),  	/* sec */
-		  RPC_INT, atoi (&ct [14]),         /* min */
-		  RPC_INT, atoi (&ct [11]),         /* hour */
-		  RPC_INT, atoi (&ct [8]),          /* mday */
-		  RPC_INT, atoi (&ct [20]),         /* year */
-		  RPC_INT, month,	                /* month */
+	    month = 11;
+	rpc_send (msock, RPC_INT, atoi (&ct[17]),	/* sec */
+		  RPC_INT, atoi (&ct[14]),	/* min */
+		  RPC_INT, atoi (&ct[11]),	/* hour */
+		  RPC_INT, atoi (&ct[8]),	/* mday */
+		  RPC_INT, atoi (&ct[20]),	/* year */
+		  RPC_INT, month,	/* month */
 		  RPC_END);
     } else {
 	long ltime = (long) time;
 	char buf[BUF_SMALL];
 
-	g_snprintf (buf, sizeof(buf), "%lx", ltime);
-	rpc_send (msock,
-		  RPC_STRING, buf,
-		  RPC_END);
+	snprintf (buf, sizeof (buf), "%lx", ltime);
+	rpc_send (msock, RPC_STRING, buf, RPC_END);
     }
 }
 
-void send_stat_info (struct stat *st)
+static void
+send_stat_info (struct stat *st)
 {
     long mylong;
     int blocks =
-#ifdef HAVE_ST_BLOCKS
+#ifdef HAVE_STRUCT_STAT_ST_BLOCKS
 	st->st_blocks;
 #else
-        st->st_size / 1024;
+	st->st_size / 1024;
 #endif
-    
-#ifdef HAVE_ST_RDEV
+
+#ifdef HAVE_STRUCT_STAT_ST_RDEV
     mylong = st->st_rdev;
 #else
     mylong = 0;
 #endif
-    rpc_send (msock, RPC_INT, (long) mylong, 
-	      RPC_INT, (long) st->st_ino,
-	      RPC_INT, (long) st->st_mode,
-	      RPC_INT, (long) st->st_nlink,
-	      RPC_INT, (long) st->st_uid,
-	      RPC_INT, (long) st->st_gid,
-	      RPC_INT, (long) st->st_size,
-	      RPC_INT, (long) blocks, RPC_END);
+    rpc_send (msock, RPC_INT, (long) mylong, RPC_INT, (long) st->st_ino,
+	      RPC_INT, (long) st->st_mode, RPC_INT, (long) st->st_nlink,
+	      RPC_INT, (long) st->st_uid, RPC_INT, (long) st->st_gid,
+	      RPC_INT, (long) st->st_size, RPC_INT, (long) blocks,
+	      RPC_END);
     send_time (msock, st->st_atime);
     send_time (msock, st->st_mtime);
     send_time (msock, st->st_ctime);
 }
 
-void do_lstat ()
+static void
+do_lstat (void)
 {
     struct stat st;
-    char   *file;
-    int    n;
+    char *file;
+    int n;
 
     rpc_get (msock, RPC_STRING, &file, RPC_END);
     n = lstat (file, &st);
@@ -344,12 +383,13 @@ void do_lstat ()
     g_free (file);
 }
 
-void do_fstat (void)
+static void
+do_fstat (void)
 {
     int handle;
     int n;
     struct stat st;
-   
+
     rpc_get (msock, RPC_INT, &handle, RPC_END);
     n = fstat (handle, &st);
     send_status (n, errno);
@@ -359,11 +399,12 @@ void do_fstat (void)
     send_stat_info (&st);
 }
 
-void do_stat ()
+static void
+do_stat (void)
 {
     struct stat st;
-    int    n;
-    char   *file;
+    int n;
+    char *file;
 
     rpc_get (msock, RPC_STRING, &file, RPC_END);
 
@@ -380,22 +421,24 @@ void do_stat ()
 
 static struct {
     int used;
-    DIR *dirs [OPENDIR_HANDLES];
-    char *names [OPENDIR_HANDLES];
+    DIR *dirs[OPENDIR_HANDLES];
+    char *names[OPENDIR_HANDLES];
 } mcfs_DIR;
 
-void close_handle (int handle)
+static void
+close_handle (int handle)
 {
-    if (mcfs_DIR.used > 0) mcfs_DIR.used--;
-    if (mcfs_DIR.dirs [handle])
-	closedir (mcfs_DIR.dirs [handle]);
-    if (mcfs_DIR.names [handle])
-	g_free (mcfs_DIR.names [handle]);
-    mcfs_DIR.dirs [handle] = 0;
-    mcfs_DIR.names [handle] = 0;
+    if (mcfs_DIR.used > 0)
+	mcfs_DIR.used--;
+    if (mcfs_DIR.dirs[handle])
+	closedir (mcfs_DIR.dirs[handle]);
+    g_free (mcfs_DIR.names[handle]);
+    mcfs_DIR.dirs[handle] = 0;
+    mcfs_DIR.names[handle] = 0;
 }
 
-void do_opendir (void)
+static void
+do_opendir (void)
 {
     int handle, i;
     char *arg;
@@ -403,37 +446,39 @@ void do_opendir (void)
 
     rpc_get (msock, RPC_STRING, &arg, RPC_END);
 
-    if (mcfs_DIR.used == OPENDIR_HANDLES){
+    if (mcfs_DIR.used == OPENDIR_HANDLES) {
 	send_status (-1, ENFILE);	/* Error */
 	g_free (arg);
 	return;
     }
-    
+
     handle = -1;
-    for (i = 0; i < OPENDIR_HANDLES; i++){
-	if (mcfs_DIR.dirs [i] == 0){
+    for (i = 0; i < OPENDIR_HANDLES; i++) {
+	if (mcfs_DIR.dirs[i] == 0) {
 	    handle = i;
 	    break;
 	}
     }
 
-    if (handle == -1){
+    if (handle == -1) {
 	send_status (-1, EMFILE);
 	g_free (arg);
 	if (!inetd_started)
-	    fprintf (stderr, "OOPS! you have found a bug in mc - do_opendir()!\n");
+	    fprintf (stderr,
+		     "OOPS! you have found a bug in mc - do_opendir()!\n");
 	return;
     }
 
-    if (verbose) printf ("handle=%d\n", handle);
+    if (verbose)
+	printf ("handle=%d\n", handle);
     p = opendir (arg);
-    if (p){
-	mcfs_DIR.dirs [handle] = p;
-	mcfs_DIR.names [handle] = arg;
-	    mcfs_DIR.used ++;
+    if (p) {
+	mcfs_DIR.dirs[handle] = p;
+	mcfs_DIR.names[handle] = arg;
+	mcfs_DIR.used++;
 
 	/* Because 0 is an error value */
-	rpc_send (msock, RPC_INT, handle+1, RPC_INT, 0, RPC_END);
+	rpc_send (msock, RPC_INT, handle + 1, RPC_INT, 0, RPC_END);
 
     } else {
 	send_status (-1, errno);
@@ -442,157 +487,176 @@ void do_opendir (void)
 }
 
 /* Sends the complete directory listing, as well as the stat information */
-void do_readdir (void)
+static void
+do_readdir (void)
 {
     struct dirent *dirent;
     struct stat st;
-    int    handle, n;
-    char   *fname = 0;
+    int handle, n;
 
     rpc_get (msock, RPC_INT, &handle, RPC_END);
-    
-    if (!handle){
+
+    if (!handle) {
 	rpc_send (msock, RPC_INT, 0, RPC_END);
 	return;
     }
 
     /* We incremented it in opendir */
-    handle --;
+    handle--;
 
-    while ((dirent = readdir (mcfs_DIR.dirs [handle]))){
+    while ((dirent = readdir (mcfs_DIR.dirs[handle]))) {
+	int fname_len;
+	char *fname;
 	int length = NLENGTH (dirent);
 
 	rpc_send (msock, RPC_INT, length, RPC_END);
 	rpc_send (msock, RPC_BLOCK, length, dirent->d_name, RPC_END);
-	fname = g_strconcat (mcfs_DIR.names [handle], 
-				PATH_SEP_STR, dirent->d_name, NULL);
+	fname_len =
+	    strlen (mcfs_DIR.names[handle]) + strlen (dirent->d_name) + 2;
+	fname = malloc (fname_len);
+	snprintf (fname, fname_len, "%s/%s", mcfs_DIR.names[handle],
+		  dirent->d_name);
 	n = lstat (fname, &st);
-	send_status (n, errno);
 	g_free (fname);
+	send_status (n, errno);
 	if (n >= 0)
 	    send_stat_info (&st);
     }
     rpc_send (msock, RPC_INT, 0, RPC_END);
 }
 
-void do_closedir (void)
+static void
+do_closedir (void)
 {
     int handle;
 
     rpc_get (msock, RPC_INT, &handle, RPC_END);
-    close_handle (handle-1);
+    close_handle (handle - 1);
 }
 
 /* }}} */
 
 /* {{{ Operations with one and two file name argument */
 
-void do_chdir (void)
+static void
+do_chdir (void)
 {
     char *file;
-    int  status;
+    int status;
 
     rpc_get (msock, RPC_STRING, &file, RPC_END);
-    
+
     status = chdir (file);
     send_status (status, errno);
     g_free (file);
 }
 
-void do_rmdir (void)
+static void
+do_rmdir (void)
 {
     char *file;
-    int  status;
+    int status;
 
     rpc_get (msock, RPC_STRING, &file, RPC_END);
-    
+
     status = rmdir (file);
     send_status (status, errno);
     g_free (file);
 }
 
-void do_mkdir (void)
+static void
+do_mkdir (void)
 {
     char *file;
-    int  mode, status;
+    int mode, status;
 
     rpc_get (msock, RPC_STRING, &file, RPC_INT, &mode, RPC_END);
-    
+
     status = mkdir (file, mode);
     send_status (status, errno);
     g_free (file);
 }
 
-void do_mknod (void)
+static void
+do_mknod (void)
 {
     char *file;
-    int  mode, dev, status;
+    int mode, dev, status;
+
+    rpc_get (msock, RPC_STRING, &file, RPC_INT, &mode, RPC_INT, &dev,
+	     RPC_END);
 
-    rpc_get (msock, RPC_STRING, &file, RPC_INT, &mode, RPC_INT, &dev, RPC_END);
-    
     status = mknod (file, mode, dev);
     send_status (status, errno);
     g_free (file);
 }
 
-void do_readlink (void)
+static void
+do_readlink (void)
 {
-    char buffer [2048];
+    char buffer[2048];
     char *file;
-    int  n;
+    int n;
 
     rpc_get (msock, RPC_STRING, &file, RPC_END);
-    n = readlink (file, buffer, 2048);
+    n = readlink (file, buffer, 2048 - 1);
     send_status (n, errno);
     if (n >= 0) {
-        buffer [n] = 0;
+	buffer[n] = 0;
 	rpc_send (msock, RPC_STRING, buffer, RPC_END);
     }
     g_free (file);
 }
 
-void do_unlink (void)
+static void
+do_unlink (void)
 {
     char *file;
-    int  status;
-    
+    int status;
+
     rpc_get (msock, RPC_STRING, &file, RPC_END);
     status = unlink (file);
     send_status (status, errno);
     g_free (file);
 }
 
-void do_rename (void)
+static void
+do_rename (void)
 {
     char *f1, *f2;
-    int  status;
-    
+    int status;
+
     rpc_get (msock, RPC_STRING, &f1, RPC_STRING, &f2, RPC_END);
     status = rename (f1, f2);
     send_status (status, errno);
-    g_free (f1); g_free (f2);
+    g_free (f1);
+    g_free (f2);
 }
 
-void do_symlink (void)
+static void
+do_symlink (void)
 {
     char *f1, *f2;
-    int  status;
-    
+    int status;
+
     rpc_get (msock, RPC_STRING, &f1, RPC_STRING, &f2, RPC_END);
     status = symlink (f1, f2);
     send_status (status, errno);
-    g_free (f1); g_free (f2);
+    g_free (f1);
+    g_free (f2);
 }
 
-void do_link (void)
+static void
+do_link (void)
 {
     char *f1, *f2;
-    int  status;
-    
+    int status;
+
     rpc_get (msock, RPC_STRING, &f1, RPC_STRING, &f2, RPC_END);
     status = link (f1, f2);
     send_status (status, errno);
-    g_free (f1); g_free (f2);
+    g_free (f1);
+    g_free (f2);
 }
 
 
@@ -600,66 +664,72 @@ void do_link (void)
 
 /* {{{ Misc commands */
 
-void do_gethome (void)
+static void
+do_gethome (void)
 {
     rpc_send (msock, RPC_STRING, (home_dir) ? home_dir : "/", RPC_END);
 }
 
-void do_getupdir (void)
+static void
+do_getupdir (void)
 {
     rpc_send (msock, RPC_STRING, (up_dir) ? up_dir : "/", RPC_END);
 }
 
-void do_chmod (void)
+static void
+do_chmod (void)
 {
     char *file;
-    int  mode, status;
-    
+    int mode, status;
+
     rpc_get (msock, RPC_STRING, &file, RPC_INT, &mode, RPC_END);
     status = chmod (file, mode);
     send_status (status, errno);
     g_free (file);
 }
 
-void do_chown (void)
+static void
+do_chown (void)
 {
     char *file;
-    int  owner, group, status;
-    
-    rpc_get (msock, RPC_STRING, &file,RPC_INT, &owner, RPC_INT,&group,RPC_END);
+    int owner, group, status;
+
+    rpc_get (msock, RPC_STRING, &file, RPC_INT, &owner, RPC_INT, &group,
+	     RPC_END);
     status = chown (file, owner, group);
     send_status (status, errno);
     g_free (file);
 }
 
-void do_utime (void)
+static void
+do_utime (void)
 {
     char *file;
-    int  status;
+    int status;
     long atime;
     long mtime;
     char *as;
     char *ms;
     struct utimbuf times;
-    
-    rpc_get (msock, RPC_STRING, &file,
-		    RPC_STRING, &as,
-		    RPC_STRING, &ms,
-		    RPC_END);
+
+    rpc_get (msock, RPC_STRING, &file, RPC_STRING, &as, RPC_STRING, &ms,
+	     RPC_END);
     sscanf (as, "%lx", &atime);
     sscanf (ms, "%lx", &mtime);
-    if (verbose) printf ("Got a = %s, m = %s, comp a = %ld, m = %ld\n",
-			  as, ms, atime, mtime);
+    if (verbose)
+	printf ("Got a = %s, m = %s, comp a = %ld, m = %ld\n", as, ms,
+		atime, mtime);
     g_free (as);
     g_free (ms);
-    times.actime  = (time_t) atime;
+    times.actime = (time_t) atime;
     times.modtime = (time_t) mtime;
     status = utime (file, &times);
     send_status (status, errno);
     g_free (file);
 }
 
-void do_quit ()
+static void
+do_quit (void)
 {
     quit_server = 1;
 }
@@ -667,11 +737,11 @@ void do_quit ()
 #ifdef HAVE_PAM
 
 struct user_pass {
-    char *username;
-    char *password;
+    const char *username;
+    const char *password;
 };
 
-int
+static int
 mc_pam_conversation (int messages, const struct pam_message **msg,
 		     struct pam_response **resp, void *appdata_ptr)
 {
@@ -679,21 +749,22 @@ mc_pam_conversation (int messages, const struct pam_message **msg,
     struct user_pass *up = appdata_ptr;
     int status;
 
-    r = g_new (struct pam_response, messages);
+    r = (struct pam_response *) malloc (sizeof (struct pam_response) *
+					messages);
     if (!r)
 	return PAM_CONV_ERR;
     *resp = r;
-    
-    for (status = PAM_SUCCESS; messages--; msg++, r++){
-	switch ((*msg)->msg_style){
 
-	case PAM_PROMPT_ECHO_ON: 
-	    r->resp = g_strdup (up->username);
-	    r->resp_retcode  = PAM_SUCCESS;
+    for (status = PAM_SUCCESS; messages--; msg++, r++) {
+	switch ((*msg)->msg_style) {
+
+	case PAM_PROMPT_ECHO_ON:
+	    r->resp = strdup (up->username);
+	    r->resp_retcode = PAM_SUCCESS;
 	    break;
 
 	case PAM_PROMPT_ECHO_OFF:
-	    r->resp = g_strdup (up->password);
+	    r->resp = strdup (up->password);
 	    r->resp_retcode = PAM_SUCCESS;
 	    break;
 
@@ -712,21 +783,22 @@ mc_pam_conversation (int messages, const struct pam_message **msg,
 }
 
 static struct pam_conv conv = { &mc_pam_conversation, NULL };
-			     
+
 
 /* Return 0 if authentication failed, 1 otherwise */
-int
-mc_pam_auth (char *username, char *password)
+static int
+mc_pam_auth (const char *username, const char *password)
 {
     pam_handle_t *pamh;
     struct user_pass up;
     int status;
-    
+
     up.username = username;
     up.password = password;
     conv.appdata_ptr = &up;
-    
-    if ((status = pam_start("mcserv", username, &conv, &pamh)) != PAM_SUCCESS)
+
+    if ((status =
+	 pam_start ("mcserv", username, &conv, &pamh)) != PAM_SUCCESS)
 	goto failed_pam;
     if ((status = pam_authenticate (pamh, 0)) != PAM_SUCCESS)
 	goto failed_pam;
@@ -736,20 +808,21 @@ mc_pam_auth (char *username, char *password)
 	goto failed_pam;
     pam_end (pamh, status);
     return 0;
-    
-failed_pam:
+
+  failed_pam:
     pam_end (pamh, status);
     return 1;
 }
 
-#else /* Code for non-PAM authentication */
-    
+#else				/* !HAVE_PAM */
+
 /* Keep reading until we find a \n */
-static int next_line (int socket)
+static int
+next_line (int socket)
 {
     char c;
 
-    while (1){
+    while (1) {
 	if (read (socket, &c, 1) <= 0)
 	    return 0;
 	if (c == '\n')
@@ -757,42 +830,52 @@ static int next_line (int socket)
     }
 }
 
-static int ftp_answer (int sock, char *text)
+static int
+ftp_answer (int sock, const char *text)
 {
-    char answer [4];
+    char answer[4];
 
     next_line (sock);
     socket_read_block (sock, answer, 3);
-    answer [3] = 0;
+    answer[3] = 0;
     if (strcmp (answer, text) == 0)
 	return 1;
     return 0;
 }
 
-int do_ftp_auth (char *username, char *password)
+static int
+send_string (int sock, const char *string)
 {
-    struct   sockaddr_in local_address;
+    return socket_write_block (sock, string, strlen (string));
+}
+
+static int
+do_ftp_auth (const char *username, const char *password)
+{
+    struct sockaddr_in local_address;
     unsigned long inaddr;
-    int      my_socket;
-    char     answer [4];
+    int my_socket;
+    char answer[4];
+
+    memset ((char *) &local_address, 0, sizeof (local_address));
 
-    bzero ((char *) &local_address, sizeof (local_address));
-    
     local_address.sin_family = AF_INET;
     /* FIXME: extract the ftp port with the proper function */
-    local_address.sin_port   = htons (21);
+    local_address.sin_port = htons (21);
 
     /*  Convert localhost to usable format */
-    if ((inaddr = inet_addr ("127.0.0.1")) != -1)
-	bcopy ((char *) &inaddr, (char *) &local_address.sin_addr,
-	       sizeof (inaddr));
-    
-    if ((my_socket = socket (AF_INET, SOCK_STREAM, 0)) < 0){
-	if (!isDaemon) fprintf (stderr, "do_auth: can't create socket\n");
+    if ((inaddr = inet_addr ("127.0.0.1")) != INADDR_NONE)
+	memcpy ((char *) &local_address.sin_addr, (char *) &inaddr,
+		sizeof (inaddr));
+
+    if ((my_socket = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
+	if (!isDaemon)
+	    fprintf (stderr, "do_auth: can't create socket\n");
 	return 0;
     }
-    if (connect (my_socket, (struct sockaddr *) &local_address,
-	     sizeof (local_address)) < 0){
+    if (connect
+	(my_socket, (struct sockaddr *) &local_address,
+	 sizeof (local_address)) < 0) {
 	fprintf (stderr,
 		 "do_auth: can't connect to ftp daemon for authentication\n");
 	close (my_socket);
@@ -801,7 +884,7 @@ int do_ftp_auth (char *username, char *password)
     send_string (my_socket, "user ");
     send_string (my_socket, username);
     send_string (my_socket, "\r\n");
-    if (!ftp_answer (my_socket, "331")){
+    if (!ftp_answer (my_socket, "331")) {
 	send_string (my_socket, "quit\r\n");
 	close (my_socket);
 	return 0;
@@ -811,7 +894,7 @@ int do_ftp_auth (char *username, char *password)
     send_string (my_socket, password);
     send_string (my_socket, "\r\n");
     socket_read_block (my_socket, answer, 3);
-    answer [3] = 0;
+    answer[3] = 0;
     send_string (my_socket, "\r\n");
     send_string (my_socket, "quit\r\n");
     close (my_socket);
@@ -820,45 +903,42 @@ int do_ftp_auth (char *username, char *password)
     return 0;
 }
 
-int do_classic_auth (char *username, char *password)
+#ifdef HAVE_CRYPT
+static int
+do_classic_auth (const char *username, const char *password)
 {
-    struct passwd *this;
-    int ret;
-#ifdef LINUX_SHADOW
+    int ret = 0;
+    const char *encr_pwd = NULL;
+    struct passwd *pw;
+#ifdef HAVE_SHADOW
     struct spwd *spw;
-    extern char *pw_encrypt (char *, char *);
-#else
-#ifdef NEED_CRYPT_PROTOTYPE
-    extern char *crypt (const char *, const char *);
 #endif
-#endif
-    
-    if ((this = getpwnam (username)) == 0)
+
+    if ((pw = getpwnam (username)) == 0)
 	return 0;
 
-#ifdef LINUX_SHADOW
+#ifdef HAVE_SHADOW
+    setspent ();
+
+    /* Password expiration is not checked! */
     if ((spw = getspnam (username)) == NULL)
-        this->pw_passwd = "*";
+	encr_pwd = "*";
     else
-        this->pw_passwd = spw->sp_pwdp;
-    if (strcmp (pw_encrypt (password, this->pw_passwd), this->pw_passwd) == 0)
-        ret = 1;
-    else
-        ret = 0;
-#else	
-#ifdef HAVE_CRYPT    
-    if (strcmp (crypt (password, this->pw_passwd), this->pw_passwd) == 0){
+	encr_pwd = spw->sp_pwdp;
+
+    endspent ();
+#else
+    encr_pwd = pw->pw_passwd;
+#endif
+
+    if (strcmp (crypt (password, encr_pwd), encr_pwd) == 0)
 	ret = 1;
-    } else
-#endif
-    {
-	ret = 0;
-    }
-#endif
+
     endpwent ();
     return ret;
 }
-#endif /* non-PAM authentication */
+#endif				/* HAVE_CRYPT */
+#endif				/* !HAVE_PAM */
 
 /* Try to authenticate the user based on:
    - PAM if the system has it, else it checks:
@@ -866,75 +946,79 @@ int do_classic_auth (char *username, char *password)
    - conventional auth (check salt on /etc/passwd, crypt, and compare
    - try to contact the local ftp server and login (if -f flag used)
 */
-int
-do_auth (char *username, char *password)
+static int
+do_auth (const char *username, const char *password)
 {
     int auth = 0;
     struct passwd *this;
-    
+
     if (strcmp (username, "anonymous") == 0)
 	username = "ftp";
 
 #ifdef HAVE_PAM
     if (mc_pam_auth (username, password) == 0)
 	auth = 1;
-#else /* if there is no pam */
+#else				/* if there is no pam */
 #ifdef HAVE_PWDAUTH
     if (pwdauth (username, password) == 0)
 	auth = 1;
     else
 #endif
+#ifdef HAVE_CRYPT
     if (do_classic_auth (username, password))
 	auth = 1;
-    else if (ftp)
+    else
+#endif
+    if (ftp)
 	auth = do_ftp_auth (username, password);
-#endif /* not pam */
-    
+#endif				/* not pam */
+
     if (!auth)
-        return 0;
-	
+	return 0;
+
     this = getpwnam (username);
     if (this == 0)
 	return 0;
 
     if (chdir (this->pw_dir) == -1)
-        return 0;
-    
-    if (this->pw_dir [strlen (this->pw_dir) - 1] == '/')
-        home_dir = g_strdup (this->pw_dir);
+	return 0;
+
+    if (this->pw_dir[strlen (this->pw_dir) - 1] == '/')
+	home_dir = strdup (this->pw_dir);
     else {
-        home_dir = g_malloc (strlen (this->pw_dir) + 2);
-        if (home_dir) {
-            strcpy (home_dir, this->pw_dir);
-            strcat (home_dir, "/");
-        } else
-            home_dir = "/";
+    	char *new_home_dir = malloc (strlen (this->pw_dir) + 2);
+	if (new_home_dir) {
+	    strcpy (new_home_dir, this->pw_dir);
+	    strcat (new_home_dir, "/");
+	    home_dir = new_home_dir;
+	} else
+	    home_dir = "/";
     }
-    	
-    
+
+
     if (setgid (this->pw_gid) == -1)
-        return 0;
-    
+	return 0;
+
 #ifdef HAVE_INITGROUPS
 #ifdef NGROUPS_MAX
     if (NGROUPS_MAX > 1 && initgroups (this->pw_name, this->pw_gid))
-        return 0;
+	return 0;
+#endif
 #endif
-#endif    
 
 #if defined (HAVE_SETUID)
     if (setuid (this->pw_uid))
-        return 0;
+	return 0;
 #elif defined (HAVE_SETREUID)
     if (setreuid (this->pw_uid, this->pw_uid))
-        return 0;
+	return 0;
 #endif
 
     /* If the setuid call failed, then deny access */
     /* This should fix the problem on those machines with strange setups */
     if (getuid () != this->pw_uid)
 	return 0;
-    
+
     if (strcmp (username, "ftp") == 0)
 	chroot (this->pw_dir);
 
@@ -943,12 +1027,13 @@ do_auth (char *username, char *password)
 }
 
 #if 0
-int do_rauth (int socket)
+static int
+do_rauth (int socket)
 {
     struct sockaddr_in from;
     struct hostent *hp;
-    
-    if (getpeername(0, (struct sockaddr *)&from, &fromlen) < 0)
+
+    if (getpeername (0, (struct sockaddr *) &from, &fromlen) < 0)
 	return 0;
     from.sin_port = ntohs ((unsigned short) from.sin_port);
 
@@ -956,37 +1041,41 @@ int do_rauth (int socket)
     if (from.sin_family != AF_INET)
 	return 0;
 
-    hp = gethostbyaddr((char *)&fromp.sin_addr, sizeof (struct in_addr),
-		       fromp.sin_family);
-    
+    hp = gethostbyaddr ((char *) &fromp.sin_addr, sizeof (struct in_addr),
+			fromp.sin_family);
+
 }
 #endif
 
-int do_rauth (int msock)
+static int
+do_rauth (int msock)
 {
     return 0;
 }
 
-void login_reply (int logged_in)
+static void
+login_reply (int logged_in)
 {
-    rpc_send (msock, RPC_INT,
-	      logged_in ? MC_LOGINOK : MC_INVALID_PASS,
+    rpc_send (msock, RPC_INT, logged_in ? MC_LOGINOK : MC_INVALID_PASS,
 	      RPC_END);
 }
 
 /* FIXME: Implement the anonymous login */
-void do_login ()
+static void
+do_login (void)
 {
     char *username;
     char *password;
-    int  result;
-    
-    rpc_get (msock, RPC_LIMITED_STRING, &up_dir, RPC_LIMITED_STRING, &username, RPC_END);
-    if (verbose) printf ("username: %s\n", username);
-    
-    if (r_auth){
+    int result;
+
+    rpc_get (msock, RPC_LIMITED_STRING, &up_dir, RPC_LIMITED_STRING,
+	     &username, RPC_END);
+    if (verbose)
+	printf ("username: %s\n", username);
+
+    if (r_auth) {
 	logged_in = do_rauth (msock);
-	if (logged_in){
+	if (logged_in) {
 	    login_reply (logged_in);
 	    return;
 	}
@@ -995,8 +1084,9 @@ void do_login ()
     rpc_get (msock, RPC_INT, &result, RPC_END);
     if (result == MC_QUIT)
 	DO_QUIT_VOID ();
-    if (result != MC_PASS){
-	if (verbose) printf ("do_login: Unknown response: %d\n", result);
+    if (result != MC_PASS) {
+	if (verbose)
+	    printf ("do_login: Unknown response: %d\n", result);
 	DO_QUIT_VOID ();
     }
     rpc_get (msock, RPC_LIMITED_STRING, &password, RPC_END);
@@ -1011,85 +1101,89 @@ void do_login ()
 
 /* This structure must be kept in synch with mcfs.h enums */
 
-struct _command {
-    char *command;
-    void (*callback)(void);
-} commands [] = {
-    { "open",       do_open },
-    { "close",      do_close },
-    { "read",       do_read },
-    { "write",      do_write },
-    { "opendir",    do_opendir }, 
-    { "readdir",    do_readdir },
-    { "closedir",   do_closedir },
-    { "stat ",      do_stat },
-    { "lstat ",     do_lstat },
-    { "fstat",      do_fstat },
-    { "chmod",      do_chmod },
-    { "chown",      do_chown },
-    { "readlink ",  do_readlink },
-    { "unlink",     do_unlink },
-    { "rename",     do_rename },
-    { "chdir ",     do_chdir },
-    { "lseek",      do_lseek },
-    { "rmdir",      do_rmdir },
-    { "symlink",    do_symlink },
-    { "mknod",      do_mknod },
-    { "mkdir",      do_mkdir },
-    { "link",       do_link },
-    { "gethome",    do_gethome },
-    { "getupdir",   do_getupdir },
-    { "login",      do_login },
-    { "quit",       do_quit },
-    { "utime",      do_utime },
-};
+static const struct _command {
+    const char *command;
+    void (*callback) (void);
+} commands[] = {
+    {
+    "open", do_open}, {
+    "close", do_close}, {
+    "read", do_read}, {
+    "write", do_write}, {
+    "opendir", do_opendir}, {
+    "readdir", do_readdir}, {
+    "closedir", do_closedir}, {
+    "stat ", do_stat}, {
+    "lstat ", do_lstat}, {
+    "fstat", do_fstat}, {
+    "chmod", do_chmod}, {
+    "chown", do_chown}, {
+    "readlink ", do_readlink}, {
+    "unlink", do_unlink}, {
+    "rename", do_rename}, {
+    "chdir ", do_chdir}, {
+    "lseek", do_lseek}, {
+    "rmdir", do_rmdir}, {
+    "symlink", do_symlink}, {
+    "mknod", do_mknod}, {
+    "mkdir", do_mkdir}, {
+    "link", do_link}, {
+    "gethome", do_gethome}, {
+    "getupdir", do_getupdir}, {
+    "login", do_login}, {
+    "quit", do_quit}, {
+"utime", do_utime}};
 
-static int ncommands = sizeof(commands)/sizeof(struct _command);
+static int ncommands = sizeof (commands) / sizeof (struct _command);
 
-void exec_command (int command)
+static void
+exec_command (int command)
 {
-    if (command < 0 ||
-	command >= ncommands ||
-	commands [command].command == 0){
+    if (command < 0 || command >= ncommands
+	|| commands[command].command == 0) {
 	fprintf (stderr, "Got unknown command: %d\n", command);
 	DO_QUIT_VOID ();
     }
-    if (verbose) printf ("Command: %s\n", commands [command].command);
-    (*commands [command].callback)();
+    if (verbose)
+	printf ("Command: %s\n", commands[command].command);
+    (*commands[command].callback) ();
 }
 
-void check_version ()
+static void
+check_version (void)
 {
     int version;
-    
+
     rpc_get (msock, RPC_INT, &version, RPC_END);
-    if (version >= 1 &&
-	version <= RPC_PROGVER)
+    if (version >= 1 && version <= RPC_PROGVER)
 	rpc_send (msock, RPC_INT, MC_VERSION_OK, RPC_END);
     else
 	rpc_send (msock, RPC_INT, MC_VERSION_MISMATCH, RPC_END);
-    
+
     clnt_version = version;
 }
 
 /* This routine is called by rpc_get/rpc_send when the connection is closed */
-void tcp_invalidate_socket (int sock)
+void
+tcp_invalidate_socket (int sock)
 {
-    if (verbose) printf ("Connection closed\n");
-    DO_QUIT_VOID();
+    if (verbose)
+	printf ("Connection closed\n");
+    DO_QUIT_VOID ();
 }
 
-void server (int sock)
+static void
+server (int sock)
 {
     int command;
-    
+
     msock = sock;
     quit_server = 0;
 
     check_version ();
     do {
-	if (rpc_get (sock, RPC_INT, &command, RPC_END) &&
-	    (logged_in || command == MC_LOGIN))
+	if (rpc_get (sock, RPC_INT, &command, RPC_END)
+	    && (logged_in || command == MC_LOGIN))
 	    exec_command (command);
     } while (!quit_server);
 }
@@ -1098,61 +1192,51 @@ void server (int sock)
 
 /* {{{ Net support code */
 
-char *get_client (int portnum)
+static const char *
+get_client (int portnum)
 {
-    int sock, clilen, newsocket;
+    int sock, newsocket;
+    unsigned int clilen;
     struct sockaddr_in client_address, server_address;
-    struct hostent *hp;
-    char hostname [255];
     int yes = 1;
-#ifdef __EMX__
-    char *me;
-#endif
-    
+
     if ((sock = socket (AF_INET, SOCK_STREAM, 0)) < 0)
-	return "Can't create socket";
+	return "Cannot create socket";
 
     /* Use this to debug: */
-    if (setsockopt (sock, SOL_SOCKET, SO_REUSEADDR, (char *) &yes, sizeof (yes)) < 0)
+    if (setsockopt
+	(sock, SOL_SOCKET, SO_REUSEADDR, (char *) &yes, sizeof (yes)) < 0)
 	return "setsockopt failed";
 
-    gethostname (hostname, 255);
-    if (verbose) printf ("hostname=%s\n", hostname);
-    hp = gethostbyname (hostname);
-#ifdef __EMX__
-    if (hp == 0 && (me = getenv("HOSTNAME")) && (0 == strcmp(hostname, me)))
-	hp = gethostbyname ("localhost");
-#endif
-    if (hp == 0)
-	return "hp = 0!";
-    
-    bzero ((char *) &server_address, sizeof (server_address));
-    server_address.sin_family = hp->h_addrtype;
+    memset ((char *) &server_address, 0, sizeof (server_address));
+    server_address.sin_family = AF_INET;
     server_address.sin_addr.s_addr = htonl (INADDR_ANY);
     server_address.sin_port = htons (portnum);
 
-    if (bind (sock, (struct sockaddr *) &server_address,
-	      sizeof (server_address)) < 0)
-	return "Can't bind";
+    if (bind
+	(sock, (struct sockaddr *) &server_address,
+	 sizeof (server_address)) < 0)
+	return "Cannot bind";
 
     listen (sock, 5);
 
-    for (;;){
+    for (;;) {
 	int child;
-	
-	clilen = sizeof (client_address);
-	newsocket = accept (sock, (struct sockaddr *) &client_address,
-			    &clilen);
 
-	if (isDaemon && (child = fork())) {
+	clilen = sizeof (client_address);
+	newsocket =
+	    accept (sock, (struct sockaddr *) &client_address, &clilen);
+
+	if (isDaemon && (child = fork ())) {
 	    int status;
-	    
+
 	    close (newsocket);
 	    waitpid (child, &status, 0);
 	    continue;
 	}
 
-	if (isDaemon && fork()) exit (0);
+	if (isDaemon && fork ())
+	    exit (0);
 
 	server (newsocket);
 	close (newsocket);
@@ -1161,27 +1245,32 @@ char *get_client (int portnum)
 }
 
 #ifdef HAVE_PMAP_SET
-void signal_int_handler (int sig)
+static void
+signal_int_handler (int sig)
 {
+    (void) sig;
+
     pmap_unset (RPC_PROGNUM, RPC_PROGVER);
 }
 #endif
 
 #ifndef IPPORT_RESERVED
-#define IPPORT_RESERVED 1024;
+#define IPPORT_RESERVED 1024
 #endif
 
-int get_port_number ()
+static int
+get_port_number (void)
 {
     int port = 0;
 
 #ifdef HAVE_RRESVPORT
     int start_port = IPPORT_RESERVED;
-    
+
     port = rresvport (&start_port);
-    if (port == -1){
-	if (geteuid () == 0){
-	    fprintf (stderr, "Could not bind the server on a reserved port\n");
+    if (port == -1) {
+	if (geteuid () == 0) {
+	    fprintf (stderr,
+		     "Cannot bind the server on a reserved port\n");
 	    DO_QUIT_NONVOID (-1);
 	}
 	port = 0;
@@ -1189,27 +1278,28 @@ int get_port_number ()
 #endif
     if (port)
 	return port;
-    
+
     port = mcserver_port;
 
     return port;
 }
 
-void register_port (int portnum, int abort_if_fail)
+static void
+register_port (int portnum, int abort_if_fail)
 {
 #ifdef HAVE_PMAP_SET
     /* Register our service with the portmapper */
     /* protocol: pmap_set (prognum, versnum, protocol, portp) */
-    
+
     if (pmap_set (RPC_PROGNUM, RPC_PROGVER, IPPROTO_TCP, portnum))
 	signal (SIGINT, signal_int_handler);
     else {
-	fprintf (stderr, "Could not register service with portmapper\n");
+	fprintf (stderr, "Cannot register service with portmapper\n");
 	if (abort_if_fail)
 	    exit (1);
     }
 #else
-    if (abort_if_fail){
+    if (abort_if_fail) {
 	fprintf (stderr,
 		 "This system lacks port registration, try using the -p\n"
 		 "flag to force installation at a given port");
@@ -1219,14 +1309,14 @@ void register_port (int portnum, int abort_if_fail)
 
 /* }}} */
 
-int main (int argc, char *argv [])
+int
+main (int argc, char *argv[])
 {
-    char *result;
-    extern char *optarg;
+    const char *result;
     int c;
 
-    while ((c = getopt (argc, argv, "fdiqp:v")) != -1){
-	switch (c){
+    while ((c = getopt (argc, argv, "fdiqp:v")) != -1) {
+	switch (c) {
 	case 'd':
 	    isDaemon = 1;
 	    verbose = 0;
@@ -1235,7 +1325,7 @@ int main (int argc, char *argv [])
 	case 'v':
 	    verbose = 1;
 	    break;
-	    
+
 	case 'f':
 	    ftp = 1;
 	    break;
@@ -1257,22 +1347,23 @@ int main (int argc, char *argv [])
 	    break;
 
 	default:
-	    fprintf (stderr, "Usage is: mcserv [options] [-p portnum]\n\n"
-		    "options are:\n"
-		    "-d  become a daemon (sets -q)\n"
-		    "-q  quiet mode\n"
-		/*    "-r  use rhost based authentication\n" */
+	    fprintf (stderr,
+		     "Usage is: mcserv [options] [-p portnum]\n\n"
+		     "options are:\n" "-d  become a daemon (sets -q)\n"
+		     "-q  quiet mode\n"
+		     /*    "-r  use rhost based authentication\n" */
 #ifndef HAVE_PAM
-		    "-f  force ftp authentication\n"
+		     "-f  force ftp authentication\n"
 #endif
-		    "-v  verbose mode\n"
-		    "-p  to specify a port number to listen\n");
+		     "-v  verbose mode\n"
+		     "-p  to specify a port number to listen\n");
 	    exit (0);
-	    
+
 	}
     }
 
-    if (isDaemon && fork()) exit (0);
+    if (isDaemon && fork ())
+	exit (0);
 
     if (portnum == 0)
 	portnum = get_port_number ();
@@ -1291,20 +1382,10 @@ int main (int argc, char *argv [])
     exit (return_code);
 }
 
-/* This functions are not used */
-void message (int is_error, char *text, char *msg)
+/* FIXME: This function should not be used in mcserv */
+void
+vfs_die (const char *m)
 {
-    printf ("%s %s\n", text, msg);
+    fputs (m, stderr);
+    exit (1);
 }
-
-char *unix_error_string (int a)
-{
-    return "none";
-}
-
-void vfs_die( char *m )
-{
-    fprintf (stderr, m);
-	exit (1);
-}
-
