@@ -37,6 +37,10 @@
 #include <signal.h>
 #include <ctype.h>	/* is_digit() */
 
+#include "../src/global.h"
+#include "../src/tty.h"		/* enable/disable interrupt key */
+#include "../src/wtools.h"	/* message() */
+#include "../src/main.h"	/* print_vfs_message */
 #include "utilvfs.h"
 #include "gc.h"
 
@@ -385,8 +389,8 @@ int mc_##name inarg \
     return result; \
 }
 
-MC_HANDLEOP(read, (int handle, char *buffer, int count), (vfs_info (handle), buffer, count) )
-MC_HANDLEOP (write, (int handle, const char *buf, int nbyte), (vfs_info (handle), buf, nbyte))
+MC_HANDLEOP(read, (int handle, void *buffer, int count), (vfs_info (handle), buffer, count) )
+MC_HANDLEOP (write, (int handle, const void *buf, int nbyte), (vfs_info (handle), buf, nbyte))
 
 
 #define MC_RENAMEOP(name) \
@@ -738,63 +742,8 @@ vfs_file_class_flags (const char *filename)
     return vfs->flags;
 }
 
-#ifdef HAVE_MMAP
-static struct mc_mmapping {
-    caddr_t addr;
-    void *vfs_info;
-    struct vfs_class *vfs;
-    struct mc_mmapping *next;
-} *mc_mmaparray = NULL;
-
-caddr_t
-mc_mmap (caddr_t addr, size_t len, int prot, int flags, int fd, off_t offset)
-{
-    struct vfs_class *vfs;
-    caddr_t result;
-    struct mc_mmapping *mcm;
-
-    if (fd == -1)
-	return (caddr_t) -1;
-    
-    vfs = vfs_op (fd);
-    result = vfs->mmap ? (*vfs->mmap)(vfs, addr, len, prot, flags, vfs_info (fd), offset) : (caddr_t)-1;
-    if (result == (caddr_t)-1){
-	errno = ferrno (vfs);
-	return (caddr_t)-1;
-    }
-    mcm =g_new (struct mc_mmapping, 1);
-    mcm->addr = result;
-    mcm->vfs_info = vfs_info (fd);
-    mcm->vfs = vfs;
-    mcm->next = mc_mmaparray;
-    mc_mmaparray = mcm;
-    return result;
-}
-
-int
-mc_munmap (caddr_t addr, size_t len)
-{
-    struct mc_mmapping *mcm, *mcm2 = NULL;
-    
-    for (mcm = mc_mmaparray; mcm != NULL; mcm2 = mcm, mcm = mcm->next){
-        if (mcm->addr == addr){
-            if (mcm2 == NULL)
-            	mc_mmaparray = mcm->next;
-            else
-            	mcm2->next = mcm->next;
-	    if (mcm->vfs->munmap)
-	        (*mcm->vfs->munmap)(mcm->vfs, addr, len, mcm->vfs_info);
-            g_free (mcm);
-            return 0;
-        }
-    }
-    return -1;
-}
-
-#endif
-
 static char *
-mc_def_getlocalcopy (struct vfs_class *vfs, const char *filename)
+mc_def_getlocalcopy (const char *filename)
 {
     char *tmp;
     int fdin, fdout, i;
@@ -846,7 +795,7 @@ mc_getlocalcopy (const char *pathname)
     struct vfs_class *vfs = vfs_get_class (path);    
 
     result = vfs->getlocalcopy ? (*vfs->getlocalcopy)(vfs, path) :
-                                 mc_def_getlocalcopy (vfs, path);
+                                 mc_def_getlocalcopy (path);
     g_free (path);
     if (!result)
 	errno = ferrno (vfs);
@@ -981,15 +930,29 @@ vfs_fill_names (fill_names_f func)
  * Returns vfs path corresponding to given url. If passed string is
  * not recognized as url, g_strdup(url) is returned.
  */
+
+static const struct {
+    const char *name;
+    size_t name_len;
+    const char *substitute;
+} url_table[] = { {"ftp://", 6, "/#ftp:"},
+                  {"mc://", 5, "/#mc:"},
+                  {"smb://", 6, "/#smb:"},
+                  {"sh://", 5, "/#sh:"},
+		  {"ssh://", 6, "/#sh:"},
+                  {"a:", 2, "/#a"}
+};
+
 char *
 vfs_translate_url (const char *url)
 {
-    if (strncmp (url, "ftp://", 6) == 0)
-	return g_strconcat ("/#ftp:", url + 6, (char *) NULL);
-    else if (strncmp (url, "a:", 2) == 0)
-	return g_strdup ("/#a");
-    else
-	return g_strdup (url);
+    size_t i;
+
+    for (i = 0; i < sizeof (url_table)/sizeof (url_table[0]); i++)
+       if (strncmp (url, url_table[i].name, url_table[i].name_len) == 0)
+           return g_strconcat (url_table[i].substitute, url + url_table[i].name_len, (char*) NULL);
+
+    return g_strdup (url);
 }
 
 int vfs_file_is_local (const char *filename)
